using System;
using System.Xml.Serialization;
using System.Data.Common;
using System.Data.OleDb;
using System.Data;
using System.Reflection;

namespace DotNetPatterns.Common
{
	/// <summary>
	/// This is main class used to implement the Abstract Packet Pattern (value object) that contains all passed business
	/// fields to each business service (ergo Facade and Product implementations).   All Facade object receive a DataSet from 
	/// the web service and translate the DataSet into a Packet that holds all data used in the business processing.  This 
	/// decouples the product classes from knowing anything about datasets and provides a single holder class so that factory methods
	/// can be used for calling any business service.  To extract data within the product simply access the Packet, which should be
	/// set by each Facade in the contructor of the appropriate ProductManager.
	/// This class contains all the methods and properties for accessing any business specific data.  This contains the original DataSet
	/// (RawData) as well as all of the delegating methods for accessing that data.  The RawData will either point directly to the orignal
	/// DataSet passed into the Facade or an enstantiated business specific data objet (e.g. PMXiPayDS) that will be accessable in the same
	/// fashion since each business specific data object must inherit from DataSet.  This provides the added flexibility of not forcing the developer
	/// into creating a lengthy business specific dataobject.  This is completely up to the developer.
	/// </summary>
	/// <remarks>
	/// 1. Deep copies not NOT yet implemented for this class
	/// 2. The translator property can be access from this class at any time to tranform Packet and DataSets
	/// 3. The meta properties (type, service, action) determine how this packet should be routed.
	///		service = determines the facade, type = determines the product, action = determines the method
	/// </remarks> 
	public class Packet : ICloneable, IComparable, IDisposable
	{
		private DataTableCollection m_oData = null;

		private DataTable m_dtMeta = null;

		/// <summary>
		/// see propety RawData
		/// </summary>
		private DataSet m_dsRawData = null;

		/// <summary>
		/// see property Type
		/// </summary>
		private string m_sType;

		/// <summary>
		/// see property Service
		/// </summary>
		private string m_sService;

		/// <summary>
		/// see property Action
		/// </summary>
		private string m_sAction;

		private string m_sServer;
		
		private long m_lTransactionID = 0;
		private long m_lProductSchemaID = 0;
		private long m_lProductIdentitySchemaID = 0;
		private string m_sProductSchemaVersion = "0";
		private string m_sIdentitySchemaVersion = "0";

		/// <summary>
		/// used to hold tranlator of packet to traslate to/from dataset
		/// </summary>
		private PacketTranslator m_oTranslator = null;

		public Packet()
		{
			RawData = new DataSet();
			Translator = new PacketTranslator();
		}


		public Packet(PacketTranslator oTranslator) : this()
		{
			m_oTranslator = oTranslator;
		}


		public object Clone()
		{
			return base.MemberwiseClone(); /// this need to change
		}


		/// <summary>
		/// To use for comparisons
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public override bool Equals(object o)
		{
			return (this.TransactionID == ((Packet)o).TransactionID);
		}

		public override int GetHashCode()
		{
			return Convert.ToInt32(TransactionID);
		}


		/// <summary>
		/// see Equals()
		/// </summary>
		/// <param name="packet1"></param>
		/// <param name="packet2"></param>
		/// <returns></returns>
		public static bool operator == (Packet packet1, Packet packet2)
		{
			if ((object)packet1 == null && (object)packet2 == null)
				return true;
			if ((object)packet1 != null && (object)packet2 == null)
				return false;
			else
				return packet1.Equals((Packet)packet2);
		}


		/// <summary>
		/// see Equals()
		/// </summary>
		/// <param name="packet1"></param>
		/// <param name="packet2"></param>
		/// <returns></returns>
		public static bool operator != (Packet packet1, Packet packet2)
		{
			if ((object)packet1 == null && (object)packet2 == null)
				return false;
			if ((object)packet1 != null && (object)packet2 == null)
				return true;
			else
				return !packet1.Equals((Packet)packet2);
		}
		

		/// <summary>
		/// Used for indexing and comparing two packets  
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public int CompareTo(object o)
		{
			Packet packet = (Packet)o;
			if ((Convert.ToInt32(this.TransactionID)) > (Convert.ToInt32(packet.TransactionID)))
				return 1;
			if ((Convert.ToInt32(this.TransactionID)) < (Convert.ToInt32(packet.TransactionID)))
				return -1;
			else
				return 0;
		}


		/// <summary>
		/// Property representing the dataset of the packet
		/// </summary>
		public DataSet RawData
		{
			get { return m_dsRawData; }
			set 
			{ 
				m_dsRawData = value;
				// reset table collection during set of rawdata
				Data = RawData.Tables;
			}
		}


		public DataTableCollection Data
		{
			get { return m_oData; }
			set { m_oData = value; }
		}


		public DataTable Meta
		{
			get { return m_dtMeta; }
			set { m_dtMeta = value; }
		}

		public string IdentitySchemaVersion
		{
			get { return m_sIdentitySchemaVersion; }
			set { m_sIdentitySchemaVersion = value; }
		}

		public string ProductSchemaVersion
		{
			get { return m_sProductSchemaVersion; }
			set { m_sProductSchemaVersion = value; }
		}

		/// <summary>
		/// The type of busiens tx, e.g. credit card, ach, etc.
		/// </summary>
		public string Type
		{
			get { return m_sType; }
			set { m_sType = value; }
		}


		/// <summary>
		/// The business action taken on this packet, e.g. authorize, settle, etc.
		/// </summary>
		public string Action
		{
			get { return m_sAction; }
			set { m_sAction = value; }
		}
		
		public string Server
		{
			get { return m_sServer; }
			set { m_sServer = value; }
		}

		/// <summary>
		/// The service this packet is used for e.g. payment, reporting
		/// </summary>
		public string Service
		{
			get { return m_sService; }
			set { m_sService = value; }
		}


		/// <summary>
		/// used to hold tranlator of packet to traslate to/from dataset
		/// </summary>
		public PacketTranslator Translator
		{
			get { return m_oTranslator; }
			set { m_oTranslator = value; }
		}


		public long TransactionID
		{
			get { return m_lTransactionID; }
			set { m_lTransactionID = value; }
		}


		public long ProductSchemaID
		{
			get { return m_lProductSchemaID; }
			set { m_lProductSchemaID = value; }
		}	
		

		public long ProductIdentitySchemaID
		{
			get { return m_lProductIdentitySchemaID; }
			set { m_lProductIdentitySchemaID = value; }
		}	

		public Packet Merge(Packet oNewPacket)
		{
			if (oNewPacket != null)
			{
				RawData.Merge(oNewPacket.RawData, true);
				Data = RawData.Tables;
			}
			return this;
		}

		// assume first table in collection
		public object GetData(string sColumn)
		{
			DataRow dr = null;
			object oReturn = null;

			if (Data[0].Rows.Count > 0)
			{
				dr = Data[0].Rows[0];
				oReturn = dr[sColumn];
				if (oReturn == System.DBNull.Value)
					oReturn = null;   // user of Packet shouldn't have to check for DBNull
			}
			return oReturn;
		}


		/// <summary>
		/// User to get particular field of packet
		/// </summary>
		public object GetData(string sTable, string sColumn)
		{
			return GetData(sTable, sColumn, 0);
		}


		/// <summary>
		/// User to get particular field of packet
		/// </summary>
		public object GetData(string sTable, string sColumn, int nRow)
		{
			DataRow dr = null;
			object oReturn = null;

			if (Data[sTable].Rows.Count > 0)
			{
				dr = Data[sTable].Rows[nRow];
				oReturn = dr[sColumn];
				
				if (oReturn == System.DBNull.Value)
					oReturn = null;  // user of Packet shouldn't have to check for DBNull
			}
			return oReturn;
		}


		/// <summary>
		/// user to retrieve an entire row as an object array for perf reasons
		/// </summary>
		/// <param name="sTable"></param>
		/// <param name="nRow"></param>
		/// <returns></returns>
		public object[] GetRowData(string sTable, int nRow)
		{
			object[] oRowArray = null;

			if (Data[sTable].Rows.Count > 0)
			{
				oRowArray = Data[sTable].Rows[nRow].ItemArray;
			}

			return oRowArray;
		}


		/// <summary>
		/// Used to set entire row using an object array for speed
		/// </summary>
		/// <param name="sTable"></param>
		/// <param name="nRow"></param>
		/// <param name="oRowArray"></param>
		public void SetRowData(string sTable, int nRow, object[] oRowArray)
		{
			Data[sTable].Rows[nRow].ItemArray = oRowArray;
			RawData.Tables[sTable].Rows[nRow].ItemArray = oRowArray;
		}


		/// <summary>
		/// User to set particular field of packet
		/// </summary>
		public void SetData(string sColumn, object oValue)
		{
			DataRow dr = null;
			
			if (Data[0].Rows.Count > 0)
			{
				dr = Data[0].Rows[0];
				dr[sColumn] = oValue;
			}
			if (RawData.Tables[0].Rows.Count > 0)
			{
				dr = RawData.Tables[0].Rows[0];
				dr[sColumn] = oValue;
			}
		}


		/// <summary>
		/// User to set particular field of packet
		/// </summary>
		public void SetData(string sTable, string sColumn, object oValue)
		{
			SetData(sTable, sColumn, 0, oValue);
		}


		/// <summary>
		/// User to set particular field of packet
		/// </summary>
		public void SetData(string sTable, string sColumn, int nRow, object oValue)
		{
			DataRow dr = null;
			
			if (Data[sTable].Rows.Count > 0)
			{
				dr = Data[sTable].Rows[nRow];
				dr[sColumn] = oValue;
			}
			if (RawData.Tables[sTable].Rows.Count > 0)
			{
				dr = RawData.Tables[sTable].Rows[nRow];
				dr[sColumn] = oValue;
			}
		}


		/// <summary>
		/// User to get/set particular field of packet
		/// </summary>
		public string this[string sColumn]
		{
			get 
			{
				return Convert.ToString(GetData(sColumn));
			}
			set 
			{
				SetData(sColumn, value);
			}
		}


		/// <summary>
		/// User to get/set particular field of packet
		/// </summary>
		public string this[string sTable, string sColumn]
		{
			get 
			{
				return Convert.ToString(GetData(sTable, sColumn));
			}
			set 
			{
				SetData(sTable, sColumn, value);
			}
		}


		/// <summary>
		/// User to get/set particular field of packet
		/// </summary>
		public string this[string sTable, string sColumn, int nRow]
		{
			get 
			{
				return Convert.ToString(GetData(sTable, sColumn, nRow));
			}
			set 
			{
				SetData(sTable, sColumn, nRow, value);
			}
		}


		/// <summary>
		/// Used to avoid casting exceptions
		/// </summary>
		/// <param name="oValue"></param>
		/// <returns></returns>
		public static string SafeCastString(object oValue)
		{
			string sReturn;

			if (oValue != null)
				sReturn = (string)oValue;
			else
				sReturn = "";
			
			return sReturn;
		}
		

		/// <summary>
		/// Used to avoid casting exceptions
		/// </summary>
		/// <param name="oValue"></param>
		/// <returns></returns>
		public static decimal SafeCastDecimal(object oValue)
		{
			decimal dReturn;

			if (oValue != null)
				dReturn = (decimal)oValue;
			else
				dReturn = (decimal)0.0;
			
			return dReturn;
		}


		/// <summary>
		/// Used to avoid casting exceptions
		/// </summary>
		/// <param name="oValue"></param>
		/// <returns></returns>
		public static int SafeCastInt(object oValue)
		{
			int nReturn;

			if (oValue != null)
				nReturn = (int)oValue;
			else
				nReturn = 0;
			
			return nReturn;
		}
		

		/// <summary>
		/// Simply delegates to the Dispose() method of the RawData (DataSet)
		/// </summary>
		public void Dispose()
		{
			RawData.Dispose();
		}

	}
}
