using System;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Net.Sockets;
using System.Net;

namespace Common
{
	/// <summary>
	/// Utilities class for common utilities operations for all pattern examples
	/// </summary>
	public class Utilities
	{
		private const string TRACE_LOG_FILE = "c:\\tracelog.txt";
		private const string TRACE_EVENTLOG_LISTENER_KEY = "Patterns.NET";
		private const string TRACE_TEXTWRITER_LISTENER_KEY = "Patterns.NET.TextWriter";
		private const string TRACE_REMOTE_LISTENER_KEY = "Patterns.NET.RemoteTracer";
		private const string EVENTLOG_SOURCE = "BaseException";

		public Utilities()
		{
		}

		/// <summary>
		/// Utility routine that takes a <CODE>short</CODE> value and chops it 
		/// into a 2 <CODE>byte</CODE>s array.
		/// </summary>
		/// <param name="value">to be chopped</param>
		/// <returns>chopped byte array</returns>
		public static byte[] ToByteArray(short value)
		{
			byte[] baBytes = new byte[2];
			baBytes[0] = (byte)(value >> 8);     // bits 1- 8
			baBytes[1] = (byte)(value & 0x00FF); // bits 9-16
			return baBytes;
		}
		
		/// <summary>
		/// Helper to build the message used for throwing exceptions
		/// </summary>
		/// <param name="e"></param>
		/// <remarks>Example: BuildErrorMessage()</remarks>
		/// <returns></returns>
		public static string BuildErrorMessage(string sMessage, string sDetail)
		{
			StringBuilder sNewMessage = new StringBuilder(sMessage);
			
			// added message text
			sNewMessage.Append(" -- ").Append(sDetail).Append("--> ");
			
			// add evironment details with call stack
			sNewMessage.Append("Environment Information:").Append("\n")
				.Append(Environment.OSVersion.Platform.ToString())
				.Append(" ").Append(Environment.OSVersion.Version.ToString()).Append("\n")
				.Append("Version Information: ").Append(Environment.Version).Append("\n")
				.Append("Current Call Stack: ").Append("\n")
				.Append(Environment.StackTrace).Append("\n").Append("\n");
			
			return sNewMessage.ToString();
		}

		/// <summary>
		/// Utility routine that takes an <CODE>int</CODE> value and chops it 
		/// into a 4 <CODE>byte</CODE>s array.
		/// </summary>
		/// <param name="value">value to be chopped</param>
		/// <returns>byte array</returns>
		public static byte[] ToByteArray(int nValue)
		{
			byte[] baBytes = new byte[4];
			baBytes[3] = (byte)( nValue               >> 24); // bits  1- 8
			baBytes[2] = (byte)((nValue & 0x00FF0000) >> 16); // bits  9-16
			baBytes[1] = (byte)((nValue & 0x0000FF00) >>  8); // bits 17-24
			baBytes[0] = (byte)( nValue & 0x000000FF       ); // bits 25-32 

			// for testing
			//byte[] baBytes2 = Encoding.ASCII.GetBytes(Convert.ToString(nValue));
			
			return baBytes;
		}

		/// <summary>
		/// Utility routine that takes an ascii <CODE>string</CODE> value and chops it 
		/// into a <CODE>byte</CODE> array.
		/// </summary>
		/// <param name="value">string to be chopped</param>
		/// <returns>byte array</returns>
		public static byte[] ToByteArray(string sValue)
		{
			byte[] abBytes = Encoding.ASCII.GetBytes(sValue);
			return abBytes;
		}
		
		/// <summary>
		/// Utility routine that takes a byte array converts it into a string.
		/// </summary>
		/// <param name="value">byte array to convert</param>
		/// <returns>byte array</returns>
		public static string FromByteArray(byte[] baBytes)
		{
			string sValue = Encoding.ASCII.GetString(baBytes);
			return sValue;
		}

		/// <summary>
		/// Adds all trace listeners
		/// </summary>
		public static void InitTraceListeners()
		{
			FileStream oTextWriter = null;

			// adding the eventlog is option since you can quickly fill up your event log in verbose mode
			if (Trace.Listeners[TRACE_EVENTLOG_LISTENER_KEY] == null)
			{
				EventLogTraceListener oEvtLogListener = new EventLogTraceListener(EVENTLOG_SOURCE);
				// ctor doesn't add the name so we have to do it here
				oEvtLogListener.Name = TRACE_EVENTLOG_LISTENER_KEY;
				Trace.Listeners.Add(oEvtLogListener);
			}
	
			if (Trace.Listeners[TRACE_TEXTWRITER_LISTENER_KEY] == null)
			{
				oTextWriter = (File.Exists(TRACE_LOG_FILE)) ? File.OpenWrite(TRACE_LOG_FILE) : File.Create(TRACE_LOG_FILE);
				Trace.Listeners.Add(new TextWriterTraceListener(oTextWriter, TRACE_TEXTWRITER_LISTENER_KEY));
			}

			// This is a custom trace listener (see PMRemoteTrace.cs) for remote tracing
			if (Trace.Listeners[TRACE_REMOTE_LISTENER_KEY] == null)
				Trace.Listeners.Add(new RemoteTraceListener(TRACE_REMOTE_LISTENER_KEY));
		}

		/// <summary>
		/// Sends any string to any host listening to any port, acknowledgement is returned as string
		/// </summary>
		/// <param name="sHost"></param>
		/// <param name="sPort"></param>
		/// <param name="sMessage"></param>
		/// <returns></returns>
		public static string SendSocketStream(string sHost, int nPort, string sMessage)
		{
			TcpClient oTcpClient = null;
			string sAck = null;
			int nBytesRead = 0;
			NetworkStream oStream = null;
				
			try
			{
				oTcpClient = new TcpClient();                
				Byte[] baRead = new Byte[100];           		       
				
				oTcpClient.Connect(sHost, nPort);
				
				// Get the stream, convert to bytes
				oStream = oTcpClient.GetStream();    
				
				// send and receive the raw bytes without a stream writer or reader
				Byte[] baSend = Encoding.ASCII.GetBytes(sMessage);
				// now send it
				oStream.Write(baSend, 0,  baSend.Length);
								
				// Read the stream and convert it to ASII			
				nBytesRead = oStream.Read(baRead, 0, baRead.Length); 
				if (nBytesRead > 0)
				{ 			
					sAck = Encoding.ASCII.GetString(baRead, 0, nBytesRead);		
				}   
        
			}
			catch(Exception ex)
			{
				throw new BaseException(null, 0, ex.Message, ex, false);					
			}
			finally
			{
				if (oStream != null)
					oStream.Close();
				if (oTcpClient != null)
					oTcpClient.Close();
			}
			return sAck;
		}
	}
}

