using System;
using System.Data;
using System.Data.SqlClient;
using System.Text;

namespace DataTierPatterns.Data
{
	/// <summary>
	///	MSSQLPolyModelData is an implementation of the abstract class PolyModelData that implements native .NET support for Mircosoft SQL 
	///	Server.  See PolyModelData for more information concerning what this class has to offer.
	/// </example>
	public class MSSQLPolyModelData : PolyModelData
	{
		/// <summary>
		/// Default constructor.
		/// Calls appropriate base ctor.
		/// </summary>
		public MSSQLPolyModelData() : base()
		{
			
		}

		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and setup a ADO.Net command object.
		/// Calls appropriate base ctor.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.</param>
		public MSSQLPolyModelData(string sSQL) : base(sSQL)
		{

		}
		
		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and setup a ADO.Net command object.
		/// Calls appropriate base ctor.
		/// </summary>
		/// <param name="sbSQL">SQL command to execute.</param>
		public MSSQLPolyModelData(StringBuilder sbSQL) : this(sbSQL.ToString())
		{
		
		}
		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and CommandType and setup a ADO.Net command object.
		/// Calls appropriate base ctor.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.</param>
		/// <param name="eType">IDbCommand CommandType</param>
		public MSSQLPolyModelData(string sSQL, CommandType eType) : base(sSQL, eType)
		{

		}

		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and CommandType and setup a ADO.Net command object.
		/// Calls appropriate base ctor.
		/// </summary>
		/// <param name="sbSQL">SQL command to execute.</param>
		/// <param name="eType">Command Type</param>
		public MSSQLPolyModelData(StringBuilder sbSQL, CommandType eType) : this(sbSQL.ToString(), eType)
		{
	
		}

		#region ADO.NET Wrapper Methods

		/// <summary>
		/// This method is creates a SqlConnection object.
		/// </summary>
		/// <returns>A IDbConnection interface reference to a ADO.NET Connection object or null if an error occurred.</returns>
		override public IDbConnection CreateConnection()
		{
			return new SqlConnection();
		}


		/// <summary>
		/// This method creates a SqlConnection object.		
		/// </summary>
		/// <param name="sConnectionString">Connection string to use during Connection object construction.</param>
		/// <returns>A IDbConnection interface reference to a ADO.NET Connection object or null if an error occurred.</returns>
		override public IDbConnection CreateConnection(string sConnectionString)
		{
			return new SqlConnection(sConnectionString);
		}
		

		/// <summary>
		/// This method creates a SqlDataAdapter object.
		/// </summary>
		/// <returns>A IDbDataAdapter interface reference to a ADO.NET DataAdapter object or null if an error occurred.</returns>
		override public IDbDataAdapter CreateDataAdapter()
		{
			return new SqlDataAdapter();
		}
			

		/// <summary>
		/// This method creates a SqlCommand object.
		/// </summary>
		/// <returns>A IDbCommand interface reference to a ADO.NET Command object or null if an error occurred.</returns>
		override public IDbCommand CreateCommand()
		{
			return new SqlCommand();
		}
	

		/// <summary>
		/// This method creates a SqlCommand object.
		/// </summary>
		/// <param name="sSQL">SQL command to use during ADO.NET Command construction</param>
		/// <returns>A IDbCommand interface reference to a ADO.NET Command object or null if an error occurred.</returns>
		override public IDbCommand CreateCommand(string sSQL)
		{
			return new SqlCommand(sSQL, (SqlConnection) Connection);
		}
	

		/// <summary>
		/// This method is adds or refreshes rows for the DataTable in the passed in DataSet.  
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sTableName">Name of DataTable to fill.</param>
		/// <returns>
		/// The number of rows successfully refreshed to the encasulated DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns> 
		override public int Fill(DataSet oDS, string sTableName)
		{
			SqlDataAdapter oDA = (SqlDataAdapter) DataAdapter;
			return oDA.Fill(oDS, sTableName);
		}


		/// <summary>
		/// Adds an SqlDataParameter to the SqlCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		override public IDbDataParameter AddDataParameter(string sName, object oDataType)
		{
			SqlCommand oCmd = null;
			oCmd = (SqlCommand) Command;
			return oCmd.Parameters.Add(sName, (SqlDbType) oDataType);
		}


		/// <summary>
		/// Adds an SqlDataParameter to the SqlCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="iSize">Gets or sets the maximum size, in bytes, of the data within the column.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		override public IDbDataParameter AddDataParameter(string sName, object oDataType, int iSize)
		{
			SqlCommand oCmd = null;
			oCmd = (SqlCommand) Command;
			return oCmd.Parameters.Add(sName, (SqlDbType) oDataType, iSize);
		}


		/// <summary>
		/// Adds an SqlDataParameter to the SqlCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="eDirection">Value indicating whether the parameter is input-only, output-only, bidirectional, or a stored procedure return value parameter.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		override public IDbDataParameter AddDataParameter(string sName, object oDataType, ParameterDirection eDirection)
		{
			SqlParameter oParm = null;
			oParm = (SqlParameter) AddDataParameter(sName, oDataType);
			if(oParm != null)
				oParm.Direction = eDirection;

			return oParm;
		}


		/// <summary>
		/// Adds an SqlDataParameter to the SqlCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="iSize">Gets or sets the maximum size, in bytes, of the data within the column.</param>
		/// <param name="eDirection">Value indicating whether the parameter is input-only, output-only, bidirectional, or a stored procedure return value parameter.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		override public IDbDataParameter AddDataParameter(string sName, object oDataType, int iSize, ParameterDirection eDirection)
		{
			SqlParameter oParm = null;
			oParm = (SqlParameter) AddDataParameter(sName, oDataType, iSize);
			if(oParm != null)
				oParm.Direction = eDirection;

			return oParm;
		}

		
		/// <summary>
		/// Returns a SqlDataParameter to the SqlCommand object.
		/// </summary>
		/// <param name="sName">Name of IDbDataParameter to return.</param>
		/// <returns>IDbDataParameter</returns>
		override public IDbDataParameter GetDataParameter(string sName)
		{
			if(Command != null)
				return (IDbDataParameter) Command.Parameters[sName];
			
			return null;
		}

		
		/// <summary>
		/// Returns the value of a SqlDataParameter in the SqlCommand object.
		/// </summary>
		/// <param name="sName">Name of IDbDataParameter.</param>
		/// <returns>IDbDataParameter</returns>
		override public object GetDataParameterValue(string sName)
		{
			SqlParameter oParm = null;
			if(Command != null)
			{
				oParm = (SqlParameter) Command.Parameters[sName];				
				if(oParm != null)
					return oParm.Value;
			}
			
			return null;
		}


		#endregion
	}
}