using System;
using System.Data;
using System.Data.SqlClient;
using DotNetPatterns.Common;
using System.Text;

namespace LCTServer
{
	/// <summary>
	/// Main driver for all aync (LCT) transaction operations
	/// </summary>
	public class TransactorFacade : ITransactor
	{
		private string m_sConnectionString = string.Empty;

		public TransactorFacade()
		{
			// for SQL Server Test 
			m_sConnectionString = "Integrated Security=SSPI;" +
				"Initial Catalog=PatternsDotNet;" + 
				"Data Source=DEV06;";
		}

		/// <summary>
		/// Calls the appropriate stored proc to increment a db count and return it
		/// </summary>
		/// <returns></returns>
		public long GenerateTransactionId()
		{
			SqlConnection cn = null;
			StringBuilder sbExecSQL = null;
			SqlCommand oCommand = null;
			long lTransactionId = 0;

			try
			{
				sbExecSQL = new StringBuilder("exec sp_DNPatternsIncCounter");
				sbExecSQL.Append(" 'JOB_ID', 1");

				// make the connection
				cn = new SqlConnection();
				cn.ConnectionString = m_sConnectionString;
				cn.Open();

				// call the storeproc sp_DNPatternsIncCounter
				if (cn.State == ConnectionState.Open )
				{
					oCommand = new SqlCommand(sbExecSQL.ToString(), cn);
					lTransactionId = (long)oCommand.ExecuteScalar();
				}
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", e.Message), false);
			}
			finally
			{
				cn.Close();
			}
			return lTransactionId;
		}

		/// <summary>
		/// This creates a new transaction 
		/// as part of a transaction table (uses the specified transactor id)
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <param name="nStatus"></param>
		/// <returns></returns>
		public DataSet CreateTransaction(long lTransactorId, int nStatus)
		{
			DataTable oTable = new DataTable();
			SqlParameter oParam = null;
			string sSelectText = string.Empty;
			SqlConnection cn = null;
			DataSet ds = null;
			SqlDataAdapter da = null;
			
			try
			{
				// make the connection
				cn = new SqlConnection();
				cn.ConnectionString = m_sConnectionString;
				cn.Open();

				if (cn.State == ConnectionState.Open )
				{
					// build the sql statement and prepare the dataset
					da = new SqlDataAdapter("SELECT * FROM JOB", cn);
					ds = new DataSet("JOB");
					da.Fill(ds, "JOB");
									
					// prepare a data adapter with insert statement
					string sInsertCommand = "INSERT INTO JOB (JOB_ID, STATUS) VALUES (@JOB_ID, @STATUS)"; 
					da.InsertCommand = new SqlCommand(sInsertCommand, cn);
				
					// bind parms to insert
					oParam = new SqlParameter("@JOB_ID",  SqlDbType.Int);
					oParam.SourceColumn = "JOB_ID";
					oParam.SourceVersion = DataRowVersion.Current;
					da.InsertCommand.Parameters.Add(oParam);
					
					oParam = new SqlParameter("@STATUS",  SqlDbType.Int);
					oParam.SourceColumn = "STATUS";
					oParam.SourceVersion = DataRowVersion.Current;
					da.InsertCommand.Parameters.Add(oParam);
					
					oTable = ds.Tables["JOB"]; 
					if (oTable != null)
					{
						// insert the new row
						DataRow oNewRow = oTable.NewRow();
						oNewRow["JOB_ID"] = lTransactorId;
						oNewRow["STATUS"] = nStatus;
						oTable.Rows.Add(oNewRow);				
				
						// save it to the database
						da.Update(ds, "JOB");
						
					}
					else
						throw new BaseException(Utilities.BuildErrorMessage("Inserting Job Failed", "Table object was null"), false);
				}
								
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", e.Message), false);
			}
			finally
			{
				da.Dispose();
				cn.Close();
			}

			return ds;
		}

		/// <summary>
		/// Updates the transaction with the dataset results 
		/// of any returned external call
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <param name="nStatus"></param>
		/// <returns></returns>
		public DataSet UpdateTransaction(long lTransactorId, int nStatus, DataSet dsOriginal)
		{
			DataTable oTable = new DataTable();
			SqlParameter oParam = null;
			string sSelectText = string.Empty;
			SqlConnection cn = null;
			SqlDataAdapter da = null;
			DataRow oRow = null;
			DataSet dsJob = null;

			try
			{
				// make the connection
				cn = new SqlConnection();
				cn.ConnectionString = m_sConnectionString;
				cn.Open();

				if (cn.State == ConnectionState.Open )
				{
					// build the sql statement and prepare the dataset
					da = new SqlDataAdapter(new StringBuilder("SELECT * FROM JOB WHERE JOB_ID = ").Append(lTransactorId).ToString(), cn);
					dsJob = new DataSet("JOB");
					da.Fill(dsJob, "JOB");
									
					// build the update sql command
					da.UpdateCommand = new SqlCommand(new StringBuilder("UPDATE JOB SET STATUS = @STATUS, DATA_SCHEMA = @DATA_SCHEMA, DATA = @DATA WHERE JOB_ID = @JOB_ID").ToString(), cn);
				
					// bind the insert parameters
					oParam = new SqlParameter("@STATUS", SqlDbType.Int);
					oParam.SourceColumn = "STATUS";
					oParam.SourceVersion = DataRowVersion.Current;
					da.UpdateCommand.Parameters.Add(oParam);
					
					oParam = new SqlParameter("@DATA_SCHEMA", SqlDbType.Image);
					oParam.SourceColumn = "DATA_SCHEMA";
					oParam.SourceVersion = DataRowVersion.Current;
					da.UpdateCommand.Parameters.Add(oParam);
					
					oParam = new SqlParameter("@DATA", SqlDbType.Image);
					oParam.SourceColumn = "DATA";
					oParam.SourceVersion = DataRowVersion.Current;
					da.UpdateCommand.Parameters.Add(oParam);
					
					oParam = new SqlParameter("@JOB_ID", SqlDbType.Int);
					oParam.SourceColumn = "JOB_ID";
					oParam.SourceVersion = DataRowVersion.Current;
					da.UpdateCommand.Parameters.Add(oParam);
					
                    da.Fill(dsJob, "JOB");

					// grab the table so we can update each field as bound by the parameters
					oTable = dsJob.Tables["JOB"]; 
					if (oTable != null)
					{
						// should be first and only row found
						oRow = oTable.Rows[0];
						if (oRow != null)
						{
							// insert the new row
							oRow["STATUS"] = nStatus;
							if (dsJob != null)
							{
								oRow["DATA_SCHEMA"] = Utilities.ToByteArray(dsOriginal.GetXmlSchema());
								oRow["DATA"] = Utilities.ToByteArray(dsOriginal.GetXml());
							}
							
							// save it to the database
							da.Update(dsJob, "JOB");
						}
						else
							throw new BaseException(Utilities.BuildErrorMessage("Updating Job Failed", "DataRow object was null"), false);
					}
					else
						throw new BaseException(Utilities.BuildErrorMessage("Updating Job Failed", "DataTable was null"), false);
				}
								
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", e.Message), false);
			}
			finally
			{
				da.Dispose();
				cn.Close();
			}

			return dsJob;
		}

		/// <summary>
		/// This will regenerate the original DataSet from the stored
		/// schema and data as xml in the job table
		/// </summary>
		/// <param name="dsJob"></param>
		/// <returns></returns>
		public DataSet ExtractOriginalData(DataSet dsJob)
		{
			DataSet dsOriginal = new DataSet("OriginalData");
			string sSchema = null;
			string sData = null;

			try
			{
				// first convert byte array to string
				if (dsJob != null)
				{
					sSchema = Utilities.FromByteArray((byte [])dsJob.Tables["JOB"].Rows[0]["DATA_SCHEMA"]);
					sData = Utilities.FromByteArray((byte [])dsJob.Tables["JOB"].Rows[0]["DATA"]);
					dsOriginal.ReadXmlSchema(new System.IO.StringReader(sSchema));
					dsOriginal.ReadXml(new System.IO.StringReader(sData));
				}
			}
			catch (Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", ex.Message), false);
			}
			
			return dsOriginal;
				
		}

		/// <summary>
		/// Returns the transaction as a dataset 
		/// for the transactor id 
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <returns></returns>
		public DataSet GetTransaction(long lTransactorId)
		{
			string sSelectText = string.Empty;
			SqlConnection cn = null;
			SqlDataAdapter da = null;
			DataSet ds = null;
			DataSet dsOriginal = null;
			
			try
			{
				// make the connection
				cn = new SqlConnection();
				cn.ConnectionString = m_sConnectionString;
				cn.Open();

				if (cn.State == ConnectionState.Open )
				{
					// build the sql statement and prepare the dataset
					da = new SqlDataAdapter(new StringBuilder("SELECT * FROM JOB WHERE JOB_ID = ").Append(lTransactorId).ToString(), cn);
					ds = new DataSet("JOB");
					da.Fill(ds, "JOB");
					
					// just so we can see the stored bytes array as returned xml data
					dsOriginal = ExtractOriginalData(ds);
				}
								
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", e.Message), false);
			}
			finally
			{
				da.Dispose();
				cn.Close();
			}

			return dsOriginal;
		}

		/// <summary>
		/// Returns the status of the transaction
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <returns></returns>
		public int GetStatus(long lTransactorId)
		{
			string sSelectText = string.Empty;
			SqlConnection cn = null;
			SqlDataAdapter da = null;
			DataSet ds = null;
			int nStatus = 0;
		
			try
			{
				// make the connection
				cn = new SqlConnection();
				cn.ConnectionString = m_sConnectionString;
				cn.Open();

				if (cn.State == ConnectionState.Open )
				{
					// build the sql statement and prepare the dataset
					da = new SqlDataAdapter(new StringBuilder("SELECT * FROM JOB WHERE JOB_ID = ").Append(lTransactorId).ToString(), cn);
					ds = new DataSet("JOB");
					da.Fill(ds, "JOB");
					
					// just so we can see the stored bytes array as returned xml data
					nStatus = (int)ds.Tables["JOB"].Rows[0]["STATUS"];
				}
								
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("General Exception thrown", e.Message), false);
			}
			finally
			{
				da.Dispose();
				cn.Close();
			}

			return nStatus;
		}

	}
}
