using System;
using System.Threading;

namespace Delays
{
	/// <summary>
	/// Will manager the process of creating a separate thread to 
	/// execute the factoring operation then notify a specified
	/// System.Windows.Forms.Control of the result of the operation.
	/// </summary>
	public class NotifyingThreadManager
	{
		/// <summary>
		/// Signature of the method that this manager can notify upon 
		/// completion of the operation.
		/// </summary>
		/// <param name="ID">A unique ID provided by the caller</param>
		/// <param name="Factored">The value factored</param>
		/// <param name="Result">The comma delimited string of factors</param>
		public delegate void CompleteEvent(int ID, 
			ulong Factored, 
			string Result);
		/// <summary>
		/// Signature of the method that this manager can notify upon 
		/// an exception occurring during execution.
		/// </summary>
		/// <param name="ID">A unique ID provided by the caller</param>
		/// <param name="Factored">The value attempted to be factored</param>
		/// <param name="ExceptionThrown">The Exception Thrown during execution</param>
		public delegate void ExceptionEvent(int ID, 
			ulong Factored, 
			Exception ExceptionThrown);

		/// <summary>
		/// Initiate the async operation and notify the control using
		/// the delegate provided.
		/// 
		/// The operation will be initiated on the Control's thread.
		/// </summary>
		/// <param name="ControlToNotify">The control to be notified</param>
		/// <param name="NotificationDelegate">The delegate to call when successful</param>
		/// <param name="ExceptionDelegate">The delegate to call when an exception occurs</param>
		public void ExecuteAsync(System.Windows.Forms.Control ControlToNotify, CompleteEvent NotificationDelegate, ExceptionEvent ExceptionDelegate)
		{
			// Store the information we will need to notify on completion
			mControlToNotify = ControlToNotify;
			mFactoringComplete = NotificationDelegate;
			mFactoringErrored = ExceptionDelegate;

			// Start the thread
			ThreadStart startThread = new ThreadStart(DoExecution);
			mExecution = new Thread(startThread);
			mExecution.Start();			
		}

		/// <summary>
		/// The Delegate we will notify on completion
		/// </summary>
		private CompleteEvent mFactoringComplete;
		/// <summary>
		/// The Delegate we will notify on error
		/// </summary>
		private ExceptionEvent mFactoringErrored;
		/// <summary>
		/// The control whose thread we will sync with
		/// </summary>
		private System.Windows.Forms.Control mControlToNotify;
		/// <summary>
		/// Our actual thread of execution
		/// </summary>
		private Thread mExecution;
		/// <summary>
		/// Exception thrown during execution (if any).
		/// </summary>
		private Exception mException = null;
		/// <summary>
		/// Exception is thrown during execution, if any.
		/// </summary>
		public Exception ExceptionThrown
		{
			get { return mException; }
		}

		/// <summary>
		/// The ID provided by the caller
		/// </summary>
		private int mID;
		/// <summary>
		/// The value to be factored.
		/// </summary>
		private ulong mToFactor;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="ID">Caller specified value to identify the result.  Not used by this class, only provided to allow caller to properly respond to the result.</param>
		/// <param name="ToFactor">The value to factor.</param>
		public NotifyingThreadManager(int ID, ulong ToFactor)
		{
			mID = ID;
			mToFactor = ToFactor;
			mException = null;
		}

		/// <summary>
		/// Our long running operation that will execute on 
		/// a separate thread.
		/// </summary>
		protected void DoExecution()
		{
			object[] paramArray = new object[3];
			try
			{
				// Get the answer
				string strFactor = Factorer.Factor(mToFactor);

				// Build the parameter array for the completion event
				paramArray[0] = mID;
				paramArray[1] = mToFactor;
				paramArray[2] = strFactor;

				// Invoke the "Success" delegate
				mControlToNotify.BeginInvoke(mFactoringComplete, paramArray);
			}
			catch(Exception ex)
			{
				// Save the exception that occured
				mException = ex;

				try
				{
					if (mFactoringErrored != null)
					{
						// Build the parameter array for the failure event
						paramArray[0] = mID;
						paramArray[1] = mToFactor;
						paramArray[2] = ex;

						// Invoke the "Failure" delegate
						mControlToNotify.BeginInvoke(mFactoringErrored, paramArray);
					}
				}
				catch(Exception ex2)
				{
					// Not much else we can do here but log and save
					// TODO: Add appropriate logging.
					mException = ex2;
				}
			}
		}
		
	}
}

