// Copyright (c) 2002-2003 Brian Eshelman, EshelTech Consulting.
// All Rights Reserved.  Developed for Dot Net Design Patterns.
// 
// Permission to use, copy, modify, and distribute this 
// software and its documentation for NON-COMMERCIAL purposes
// and without fee is hereby granted provided that this 
// copyright notice appears in all copies. 
//
// THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT 
// THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR 
// NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY 
// DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, 
// MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
// Please send any comments to beshelman@esheltech.com
//

using System;
using System.Threading;

namespace TestBed
{
	/// <summary>
	/// Will manager the process of creating a separate thread to 
	/// execute the factoring operation then notify a specified
	/// System.Windows.Forms.Control of the result of the operation.
	/// </summary>
	public class NotifyingThreadManager
	{
		/// <summary>
		/// Signature of the method that this manager can notify upon 
		/// completion of the operation.
		/// </summary>
		public delegate void CompleteEvent(bool Return, 
			System.Collections.SortedList ParamList);
		/// <summary>
		/// Signature of the method that this manager can notify upon 
		/// an exception occurring during execution.
		/// </summary>
		public delegate void ExceptionEvent(System.Collections.SortedList ParamList,
			Exception ExceptionThrown);

		/// <summary>
		/// Initiate the async operation and notify the control using
		/// the delegate provided.
		/// 
		/// The operation will be initiated on the Control's thread.
		/// </summary>
		/// <param name="ControlToNotify">The control to be notified</param>
		/// <param name="NotificationDelegate">The delegate to call when successful</param>
		/// <param name="ExceptionDelegate">The delegate to call when an exception occurs</param>
		public void ExecuteAsync(System.Windows.Forms.Control ControlToNotify, CompleteEvent NotificationDelegate, ExceptionEvent ExceptionDelegate)
		{
			// Store the information we will need to notify on completion
			mControlToNotify = ControlToNotify;
			mFactoringComplete = NotificationDelegate;
			mFactoringErrored = ExceptionDelegate;

			// Start the thread
			ThreadStart startThread = new ThreadStart(DoExecution);
			mExecution = new Thread(startThread);
			mExecution.Start();			
		}

		/// <summary>
		/// The Delegate we will notify on completion
		/// </summary>
		private CompleteEvent mFactoringComplete;
		/// <summary>
		/// The Delegate we will notify on error
		/// </summary>
		private ExceptionEvent mFactoringErrored;
		/// <summary>
		/// The control whose thread we will sync with
		/// </summary>
		private System.Windows.Forms.Control mControlToNotify;
		/// <summary>
		/// Our actual thread of execution
		/// </summary>
		private Thread mExecution;
		/// <summary>
		/// Exception thrown during execution (if any).
		/// </summary>
		private Exception mException = null;
		/// <summary>
		/// Exception is thrown during execution, if any.
		/// </summary>
		public Exception ExceptionThrown
		{
			get { return mException; }
		}

		/// <summary>
		/// The Parameter LIst provided by the caller
		/// </summary>
		private System.Collections.SortedList mParameterList;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="ID">Caller specified value to identify the result.  Not used by this class, only provided to allow caller to properly respond to the result.</param>
		/// <param name="ToFactor">The value to factor.</param>
		public NotifyingThreadManager(System.Collections.SortedList ParameterList)
		{
			mParameterList = ParameterList;
			mException = null;
		}

		/// <summary>
		/// Our long running operation that will execute on 
		/// a separate thread.
		/// </summary>
		protected void DoExecution()
		{
			object[] paramArray = new object[2];
			try
			{
				TestBed.ITestable iface = GetTestObject(mParameterList);

				// Get the answer
				bool bRet = iface.Test(ref mParameterList);

				// Build the parameter array for the completion event
				paramArray[0] = bRet;
				paramArray[1] = mParameterList;

				// Invoke the "Success" delegate
				mControlToNotify.BeginInvoke(mFactoringComplete, paramArray);
			}
			catch(Exception ex)
			{
				// Save the exception that occured
				mException = ex;

				try
				{
					if (mFactoringErrored != null)
					{
						// Build the parameter array for the failure event
						paramArray[0] = mParameterList;
						paramArray[1] = ex;

						// Invoke the "Failure" delegate
						mControlToNotify.BeginInvoke(mFactoringErrored, paramArray);
					}
				}
				catch(Exception ex2)
				{
					// Not much else we can do here but log and save
					// TODO: Add appropriate logging.
					mException = ex2;
				}
			}

		}
		
		private TestBed.ITestable GetTestObject(System.Collections.SortedList ParameterList)
		{
			System.Reflection.Assembly LoadedAssembly = System.Reflection.Assembly.LoadFrom(ParameterList["Assembly"] as string);
			System.Type type = LoadedAssembly.GetType(ParameterList["Class"] as string);
			if (type == null)
				return null;

			TestBed.ITestable iface = type.GetConstructor(System.Type.EmptyTypes).Invoke(System.Type.EmptyTypes) as TestBed.ITestable;
			return iface;
		}
	}
}

