using System;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Runtime.Serialization;

namespace RemoteTracer
{
	/// <summary>
	/// The is the base exception class for Patterns.NET
	/// </summary>
	[Serializable]
	public class BaseException : System.ApplicationException
	{
		/// <summary>
		/// One of the error codes. 
		/// </summary>
		private int m_nCode;
		
		/// <summary>
		/// Self explanation, shouldn't be used much
		/// </summary>
		public BaseException(){;}

		/// <summary>
		/// Same as following ctor except this is no code
		/// </summary>
		/// <param name="sMessage"></param>
		public BaseException(string sMessage, bool bLog) : this(sMessage, null, bLog){;}
		
		/// <summary>
		/// Allow inner exception to be created to create exception chaining
		/// </summary>
		/// <param name="sMessage"></param>
		/// <param name="oInnerException"></param>
		public BaseException(string sMessage, System.Exception oInnerException, bool bLog) : this(null, 0, sMessage, oInnerException, bLog){;}
		
		/// <summary>
		/// Constructs a new instance of this class that 
		/// will carry the Code received from originating error.
		/// </summary>
		/// <param name="oSource"><CODE>Object</CODE> that throwed this <CODE>Exception</CODE>.</param>
		/// <param name="nCode">one of the error codes.</param>
		/// <param name="sMessage">original reply from the exception that trigered this <CODE>Exception</CODE>.</param>
		/// <remarks></remarks>
		public BaseException(object oSource, int nCode, string sMessage, bool bLog) : this(oSource, nCode, sMessage, null, bLog){;}
		
		/// <summary>
		/// Constructs a new instance of this class that 
		/// will carry the Code and 
		/// the original error received as well as chain the inner exception class
		/// </summary>
		/// <param name="oSource"><CODE>Object</CODE> that throwed this <CODE>Exception</CODE>.</param>
		/// <param name="nCode">one of the error codes.</param>
		/// <param name="sMessage">original reply from the exception that triggered this <CODE>Exception</CODE>.</param>
		/// <param name="oInnerException">The exception one up on the chain</param>
		public BaseException(object oSource, int nCode, string sMessage, System.Exception oInnerException, bool bLog) : base(sMessage, oInnerException)
		{
			if (oSource != null)
				base.Source = oSource.ToString();
			Code = nCode;

			// need to add logic to check what log destination we should loging to e.g. file, eventlog, database, remote debugger
			if (bLog)
			{
				// log it
				Dump(Format(oSource, nCode, sMessage, oInnerException));
			}
		}

		/// <summary>
		/// Ctor used during serialization, any custom data members should go here
		/// </summary>
		/// <param name="oInfo"></param>
		/// <param name="oContext"></param>
		protected BaseException(SerializationInfo oInfo, StreamingContext oContext) : base(oInfo, oContext)
		{
			m_nCode = oInfo.GetInt32("m_nCode");
		}

		/// <summary>
		/// Called during serialization, any custom data member you wish to serialize should go here
		/// </summary>
		/// <param name="oInfo"></param>
		/// <param name="oContext"></param>
		public override void GetObjectData(SerializationInfo oInfo, StreamingContext oContext)
		{
			oInfo.AddValue("m_nCode", m_nCode, typeof(int));
			base.GetObjectData(oInfo, oContext);
		}

		/// <summary>
		/// Writes the entire message to all trace listeners including the event log
		/// </summary>
		/// <param name="oSource"></param>
		/// <param name="nCode"></param>
		/// <param name="sMessage"></param>
		/// <param name="oInnerException"></param>
		private void Dump(string sMessage)
		{
			// write to all trace listeners
			Trace.WriteLineIf(Config.TraceLevel.TraceError, sMessage);
				
			// see PMUtilities.InitTraceListeners()
			// record it to the event log if error tracing is on
			// The EventLog trace listener wasn't added to the collection to prevent further traces
			// from being sent to the event log to avoid filling up the eventlog
			if (Config.TraceLevel.TraceError)
				EventLog.WriteEntry("BaseException", sMessage);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="oSource"></param>
		/// <param name="nCode"></param>
		/// <param name="sMessage"></param>
		/// <param name="oInnerException"></param>
		/// <returns></returns>
		public static string Format(object oSource, int nCode, string sMessage, System.Exception oInnerException)
		{
			StringBuilder sNewMessage = new StringBuilder();
			string sErrorStack = null;
			
			// get the error stack, if InnerException is null, sErrorStack will be "exception was not chained" and should never be null
			sErrorStack = BuildErrorStack(oInnerException);
				
			// we want immediate gradification 
			Trace.AutoFlush = true;

			sNewMessage.Append("Exception Summary \n")
				.Append("--------------------------------------------\n")
				.Append(DateTime.Now.ToShortDateString())
				.Append(":")
				.Append(DateTime.Now.ToShortTimeString())
				.Append(" - ")
				.Append(sMessage)
				.Append("\n\n")
				.Append(sErrorStack);

			return sNewMessage.ToString();
		}

		/// <summary>
		/// Returns the code produced by the 
		/// whatever triggered this <CODE>Exception</CODE>.
		/// </summary>
		/// <returns>error code</returns>
		public int Code 
		{
			get {return m_nCode;}
			set {m_nCode = value;}
		}
		
		/// <summary>
		/// Takes a first nested exception object and builds a error stack from its chained contents
		/// </summary>
		/// <param name="oChainedException"></param>
		/// <returns></returns>
		private static string BuildErrorStack(System.Exception oChainedException)
		{
			string sErrorStack = null;
			StringBuilder sbErrorStack = new StringBuilder();
			int nErrStackNum = 1;
			System.Exception oInnerException = null;
			string sAppName = null;
			string sMethodName = null;

			if (oChainedException != null)
			{
				sbErrorStack.Append("Exception Details --- Error Stack\n")
					.Append("-------------------------------------------\n");
				
				oInnerException = oChainedException;
				while (oInnerException != null)
				{
					sAppName = (oInnerException.TargetSite != null) ? oInnerException.TargetSite.Name : "<not set>";
					sMethodName = (oInnerException.Source != null) ? oInnerException.Source : "<not set>";
					
					sbErrorStack.Append(nErrStackNum)
						.Append(") ")
						.Append("Method: ")
						.Append(sAppName)
						.Append("\n    Assembly: ")
						.Append(sMethodName)
						.Append("\n    Message: \n    ")
						.Append(oInnerException.Message)
						.Append("\n\n");
					
					oInnerException = oInnerException.InnerException;
					nErrStackNum++;
				}
				
				sbErrorStack.Append("\nCall Stack\n")
					.Append("-------------------------------------------\n")
					.Append(oChainedException.StackTrace);
					
				sErrorStack = sbErrorStack.ToString();				
			}
			else
			{
				sErrorStack = "exception was not chained";
			}

			return sErrorStack;
		}

	}
}



