using System;
using System.Diagnostics;
using System.Resources;
using System.Reflection;
using System.Configuration;
using System.Xml.Serialization;
using System.Text;
using System.IO;
using System.Collections;
using System.Xml;
using System.Collections.Specialized;

namespace RemoteTracer
{
	/// <summary>
	/// This represents all dynamic configuration data for paymetric
	/// </summary>
	public class Config
	{
		/// <summary>
		/// The tracing level is determined by the trace switch which is
		/// accessable via the Config.TraceLevel static data member, to send remote traces (if turned on), the clent
		/// must use the following code template:
		/// 
		///		Trace.WriteLineIf(Config.TraceLevel.TraceVerbose, "starting packet translation...");
		///		
		/// </summary>
		/// <example>Trace.WriteLineIf(Config.TraceLevel.TraceVerbose, "starting packet translation...");</example> 
		/// <remarks>
		/// To change the trace level you must edit the launching applications configuration file as such:
		/// The name attribute must match the name given to the trace extension, e.g. RemoteTraceLevel
		/// <system.diagnostics>
		///		<switches>
		///			<add name="RemoteTraceLevel" value="1" />
		///		</switches>
		/// </system.diagnostics>
		/// 0 (off), 1 (error), 2 (warning), 3 (info), OR 4 (verbose)
		/// </remarks>
		public static TraceSwitch TraceLevel = new TraceSwitch("RemoteTraceLevel", "Remote Tracing Level");

		/// <summary>
		/// The tracing level is determined by the trace switch which is
		/// accessable via the PMConfig.TraceLevel static data member, to send remote traces (if turned on), the clent
		/// must use the following code template:
		/// 
		///		Trace.WriteLineIf(PMConfig.TraceLevel.TraceVerbose, "starting packet translation...");
		///		
		/// </summary>
		/// <example>Trace.WriteLineIf(PMConfig.TraceLevel.TraceVerbose, "starting packet translation...");</example> 
		/// <remarks>
		/// To change the trace level you must edit the launching applications configuration file as such:
		/// The name attribute must match the name given to the trace extension, e.g. PaymetricTraceLevel
		/// <system.diagnostics>
		///		<switches>
		///			<add name="PaymetricTraceLevel" value="1" />
		///		</switches>
		/// </system.diagnostics>
		/// 0 (off), 1 (error), 2 (warning), 3 (info), OR 4 (verbose)
		/// </remarks>
		private TraceSwitch m_oCurrentTraceSwitch;

		/// <summary>
		/// </summary>
		public TraceSwitch CurrentTraceSwitch
		{
			get {return m_oCurrentTraceSwitch;}
			set {m_oCurrentTraceSwitch = value;}
		}
		
		/// <summary>
		/// for testing currently
		/// </summary>
		public class EnvironmentSettings
		{
			public ServerInfo[] Servers;
		}

		/// <summary>
		/// for testing currently
		/// </summary>
		public class ServerInfo
		{
			public string MachineName;
			public string OS; 
		}

		/// <summary>
		/// Returns the set server settings for this application
		/// Cannot be called from a web service if web.config is in the application directory
		/// </summary>
		/// <returns></returns>
		public static IDictionary GetServerInfo()
		{
			IDictionary oTable = null;

			oTable = (IDictionary) ConfigurationSettings.GetConfig("serverInfo");
			return oTable;
		}


		/// <summary>
		/// self reference used for singleton config object
		/// </summary>
		private static Config m_oConfig;

		/// <summary>
		/// Singleton ctor
		/// </summary>
		protected Config()
		{
			CurrentTraceSwitch = new TraceSwitch("DotNetPatternsTraceSwitch", "Trace Switch for Demo only");
		}
		
		/// <summary>
		/// Singleton getter
		/// </summary>
		/// <returns></returns>
		public static Config GetConfig()
		{
			if (m_oConfig == null)
				m_oConfig = new Config();
			return m_oConfig;
		}
	
		/// <summary>
		/// Shortcut for obtaining travelevel from PMConfig singleton, avoids long argument
		/// </summary>
		/// <returns></returns>
		public static bool TraceVerbose
		{
			get { return Config.GetConfig().CurrentTraceSwitch.TraceVerbose;}
		}

		/// <summary>
		/// Shortcut for obtaining travelevel from PMConfig singleton, avoids long argument
		/// </summary>
		/// <returns></returns>
		public static bool TraceError
		{
			get { return Config.GetConfig().CurrentTraceSwitch.TraceError;}
		}

		/// <summary>
		/// Shortcut for obtaining travelevel from PMConfig singleton, avoids long argument
		/// </summary>
		/// <returns></returns>
		public static bool TraceWarning
		{
			get {return Config.GetConfig().CurrentTraceSwitch.TraceWarning;}
		}

		/// <summary>
		/// Shortcut for obtaining travelevel from PMConfig singleton, avoids long argument
		/// </summary>
		/// <returns></returns>
		public static bool TraceInfo
		{
			get {return Config.GetConfig().CurrentTraceSwitch.TraceInfo;}
		}

		/// <summary>
		/// Retrieves and application setting using the specified name and
		/// returns the set string
		/// </summary>
		/// <param name="sName"></param>
		/// <returns></returns>
		public static string GetAppSetting(string sName)
		{
			return ConfigurationSettings.AppSettings[sName];
		}
        		
	}
}
