using System;
using System.Messaging;
using System.Runtime.Serialization;
using System.Reflection;

namespace Common
{
	/// <summary>
	/// This is message information send/received from a queue
	/// For example, for FTP file transfers the Data property will contain the actual file contents
	/// </summary>
	public struct BusinessMessage
	{
		/// <summary>
		/// see properties for each data member
		/// </summary>
		private string sType;
		private string sURI;
		private byte[] baData;
		private string sFileName;
		private string sUserId;
		private string sPassword;
		private string sQueueName;
		private string sMessageText;
		private string sDate;
		private string sTime;

		/// <summary>
		///	The type of message, e.g. "FTP", setting the type sets the queuename 
		/// </summary>
		public string MessageType
		{
			get {return sType;}
			set 
			{
				sType = value;
				sQueueName = ".\\private$\\patterns.net_" + sType.ToLower();
			}
		}

		/// <summary>
		///	The string text of a message 
		/// </summary>
		public string MessageText
		{
			get {return sMessageText;}
			set	{sMessageText = value;}
		}

		/// <summary>
		///	A simple string date 
		/// </summary>
		public string Date
		{
			get {return sDate;}
			set	{sDate = value;}
		}

		/// <summary>
		///	A simple string time
		/// </summary>
		public string Time
		{
			get {return sTime;}
			set	{sTime = value;}
		}

		/// <summary>
		///	For messages such as FTP file transfer this is the uri of the ftp server 
		/// </summary>
		public string Uri
		{
			get {return sURI;}
			set {sURI = value;}
		}

		/// <summary>
		/// The Data containing the main contents of the message, for FTP this is the file contents
		/// </summary>
		public byte[] Data
		{
			get {return baData;}
			set {baData = value;}
		}

		/// <summary>
		/// Used for messages such as FTP file transfers, represents the filename that will be sent to an 
		/// FTP server when the message is dequeued
		/// </summary>
		public string FileName
		{
			get {return sFileName;}
			set {sFileName = value;}
		}

		/// <summary>
		/// Used for messages such as FTP file transfers, represents the user id that will be sent to an 
		/// FTP server when the message is dequeued
		/// </summary>
		public string UserId
		{
			get {return sUserId;}
			set {sUserId = value;}
		}

		/// <summary>
		/// Used for messages such as FTP file transfers, represents the password that will be sent to an 
		/// FTP server when the message is dequeued
		/// </summary>
		public string Password
		{
			get {return sPassword;}
			set {sPassword = value;}
		}

		/// <summary>
		/// This is set when the messageinfo type is set, there will be one queue per message type
		/// e.g. for FTP file transfers a queue name of \private$\Paymetric.Net_FTP will be used
		/// </summary>
		public string QueueName
		{
			get {return sQueueName;}
		}
	}

	/// <summary>
	/// Used for sending asynchronous messages to a durable message queue of some kind
	/// This currently using MSMQ and assumes it is installed, eventually this will be implemented
	/// to use any queuing framework.
	/// </summary>
	public class Messenger
	{
		/// <summary>
		/// Data member for the main message information to be sent and received.
		/// </summary>
		private BusinessMessage oMessage;

		/// <summary>
		/// Property for the message information structure, which is an inner structure
		/// </summary>
		public BusinessMessage MessageInfo
		{
			get {return oMessage;}
			set {oMessage = value;}
		}
		
		/// <summary>
		/// Initializes an empty message information structure
		/// </summary>
		public Messenger()
		{
			MessageInfo = new BusinessMessage();
		}

		/// <summary>
		/// Sends the providing message info structure to a queue, queuename should be set in the MessageInfo struct
		/// This just set the MessageInfo property and delegates to Send()
		/// </summary>
		/// <param name="oMessage"></param>
		public void Send(BusinessMessage oMessage)
		{
			MessageInfo = oMessage;
			Send();
		}

		/// <summary>
		/// Sends the set MessageInfo data to the queue based on the MessageInfo.QueueName property
		/// This will be serialized as xml in the message body
		/// </summary>
		public void Send()
		{
			try
			{
				string sQueuePath = MessageInfo.QueueName;
				if (!MessageQueue.Exists(sQueuePath))
				{
					// queue doesn't exist so create
					MessageQueue.Create(sQueuePath);
				}
				// Init the queue
				MessageQueue oQueue = new MessageQueue(sQueuePath);
				// send the message
				oQueue.Send(MessageInfo, MessageInfo.MessageType + " - " + MessageInfo.Uri);
			}
			catch (Exception e)
			{
				throw new BaseException(this, 0, e.Message, e, false);
			}

		}

		/// <summary>
		/// Receives a message based on the MessageInfo.QueueName of the MessageInfo struct passed in.
		/// </summary>
		/// <param name="oMessage"></param>
		public BusinessMessage Receive(BusinessMessage oMessage, int nTimeOut)
		{
			MessageInfo = oMessage;
			return Receive(nTimeOut);
		}

		/// <summary>
		/// Uses the set MessageInfo.QueueName to retrieve a message from the specified queue.
		/// If the queue cannot be found or a matching BusinessMessage is not in the queue an exception will be thrown
		/// This is a "polling" action of receiving a message from the queue.
		/// </summary>
		/// <returns>A BusinessMessage contains the body of the message - deserialized from the message body xml</returns>
		public BusinessMessage Receive(int nTimeOut)
		{
			try
			{
				string sQueuePath = MessageInfo.QueueName;
				if (!MessageQueue.Exists(sQueuePath))
				{
					// queue doesn't exist so throw exception
					throw new Exception("Receive - Error " + sQueuePath + " queue does not exist.");
				}

				// Init the queue
				MessageQueue oQueue = new MessageQueue(sQueuePath);
				
				// IMPORTANT: crucial that inner classes/structs must be derefenced by the + sign and not a period (.)
				//string[] saTypes = {"DotNetPatterns.Common.BusinessMessage, " + Assembly.GetExecutingAssembly().ToString()};
				//((XmlMessageFormatter)oQueue.Formatter).TargetTypeNames = saTypes;
				((XmlMessageFormatter)oQueue.Formatter).TargetTypes = new Type[]{typeof(BusinessMessage)};

				// receive the message, timeout in only 5 seconds -- TODO: this should probably change later
				System.Messaging.Message oRawMessage = oQueue.Receive(new TimeSpan(0, 0, nTimeOut));

				// extract the body and cast it to our BusinessMessage type so we can return it
				BusinessMessage oMessageBody = (BusinessMessage)oRawMessage.Body;
				MessageInfo = oMessageBody;

				return oMessageBody;
			}
			catch (Exception e)
			{
				throw new BaseException(this, 0, e.Message, e, false);
			}
	
		}
		
	}
}