using System;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Diagnostics;
using DotNetPatterns.Common;

namespace DataTierPatterns.Data
{
	public class MSSQLPolyModel : PolyModel
	{
		public MSSQLPolyModel() : base()
		{

		}


		/// <summary>
		/// This will take the name of a product schema and 
		/// return a long value that is the Schema ID for the current XSD Schema for a product.
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <returns>ID of schema or -1 if an error is encountered</returns>
		public override long GetCurrentSchemaIDForProductSchema(string sProductName)
		{
			try
			{
                Profiler.BeginInstrumentation();

				//	Check argument
				if(sProductName.Length == 0)
					return DTPConstants.DB_ERROR;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return DTPConstants.DB_ERROR;

				//	Define the query to execute
				sbSQL.Append(DTPConfig.GetSQLText(106));
				sbSQL.Append(DTPConfig.GetSQLText(107));
				sbSQL.Append(DTPConfig.GetSQLText(108));
				sbSQL.AppendFormat(DTPConfig.GetSQLText(109), sProductName);
				sbSQL.AppendFormat(DTPConfig.GetSQLText(110), Convert.ToInt32(DTPConstants.SCHEMA_TYPE.PRODUCT));

				PolyModelData oDB = null;
				oDB =  ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return DTPConstants.DB_ERROR;

				//	Get the Schema ID for the current product
				long lSchemaID = DTPConstants.DB_ERROR;
				lSchemaID = Convert.ToInt32(oDB.ExecuteScalar(sbSQL));

				return lSchemaID;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return DTPConstants.DB_ERROR;
		}


		/// <summary>
		/// This method will take the name of a product schema and return the ID of the current identity 
		/// schema associated with the product schema.  
		/// </summary>
		/// <param name="lProductSchemaID">Product Name</param>
		/// <returns>Identity Schema ID or DTPConstants.DB_ERROR if an error occurs</returns>
		public override long GetCurrentLookupSchemaID(long lProductSchemaID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lProductSchemaID == 0)
					return DTPConstants.DB_ERROR;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return DTPConstants.DB_ERROR;

				//	Define the query to execute
				sbSQL.Append(DTPConfig.GetSQLText(111));
				sbSQL.Append(DTPConfig.GetSQLText(112));
				sbSQL.Append(DTPConfig.GetSQLText(113));
				sbSQL.AppendFormat(DTPConfig.GetSQLText(114), lProductSchemaID);
				sbSQL.AppendFormat(DTPConfig.GetSQLText(115), Convert.ToInt32(DTPConstants.SCHEMA_TYPE.IDENTITY));

				PolyModelData oDB = null;
				oDB =  ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return DTPConstants.DB_ERROR;

				//	Get the Schema ID for the current product
				long lSchemaID = DTPConstants.DB_ERROR;
				lSchemaID = Convert.ToInt32(oDB.ExecuteScalar(sbSQL));

				return lSchemaID;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return DTPConstants.DB_ERROR;
		}


		/// <summary>
		/// This method will take the name of a product and a schema type and return an XSD Schema for a DataSet that matches the
		/// search criteria.
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <param name="eType">Schema Type</param>
		/// <returns>Packet hydrated from the XSD Schema of a DataSet</returns>
		public override Packet GetCurrentSchema(string sProductName, DTPConstants.SCHEMA_TYPE eType)
		{			
			Packet oOutPacket = null;
			string sVersion = "0";

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(sProductName.Length == 0)
					return null;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return null;

				//	Define the query to execute
				sbSQL.Append(DTPConfig.GetSQLText(116));
				sbSQL.Append(DTPConfig.GetSQLText(117));
				sbSQL.Append(DTPConfig.GetSQLText(118));
				sbSQL.AppendFormat(DTPConfig.GetSQLText(119), sProductName);
				sbSQL.AppendFormat(DTPConfig.GetSQLText(120), Convert.ToInt32(eType));
	
				string sXMlSchema = string.Empty;
				sXMlSchema = GetSchemaBySQL(sbSQL.ToString(), out sVersion);
				if(sXMlSchema.Length == 0)									
					return null;
				
				//	Return the hydrated schema
				oOutPacket = XML2Packet(sXMlSchema);

				if (eType == DTPConstants.SCHEMA_TYPE.IDENTITY)
					oOutPacket.IdentitySchemaVersion = sVersion;
				else
					oOutPacket.ProductSchemaVersion = sVersion;

				return oOutPacket;				
			}
			catch(Exception e)
			{				
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This method will take the passed in schema arguments and 
		/// return a DataSet populated with a XSD schema for a DataSet.
		/// </summary>
		/// <param name="sSchemaName">Schema SchemaName</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="iType">Schema Type</param>
		/// <returns>string populated with XSD schema.</returns>
		public override string GetSchemaAsString(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{			
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(sSchemaName.Length == 0)
					return string.Empty;
				
				//	Check argument
				if(sVersion.Length == 0)
					return string.Empty;

				//	Check argument
				if(eType == DTPConstants.SCHEMA_TYPE.UNKNOWN)
					return string.Empty;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return string.Empty;

				//	Define the query to execute				
				sbSQL.Append("execute sp_executesql ");
				sbSQL.Append("N'SELECT XML_DATA FROM dbo.[SCHEMA] WHERE NAME = @SchemaName AND VERSION = @SchemaVersion AND SCHEMA_TYPE = @SchemaType', ");
				sbSQL.Append("N'@SchemaName VARCHAR(65), @SchemaVersion VARCHAR(32), @SchemaType BIGINT', ");
				sbSQL.AppendFormat("@SchemaName ='{0}', @SchemaVersion = '{1}', @SchemaType = {2} ", sSchemaName, sVersion, Convert.ToInt32(eType));				
				
				return GetSchemaBySQL(sbSQL.ToString());
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return string.Empty;
		}


		/// <summary>
		/// This method will take the ID of a SCHEMA and return a string populated with the XSD schema for a DataSet.
		/// </summary>
		/// <param name="lSchemaID">Schema ID</param>
		/// <returns>string populated with an XSD Schema for a DataSet; or a empty string if an error occurred.</returns>
		public override string GetSchemaAsString(long lSchemaID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lSchemaID == 0)
					return string.Empty;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return string.Empty;

				//	Define the query to execute
				sbSQL.Append(DTPConfig.GetSQLText(125));
				sbSQL.Append(DTPConfig.GetSQLText(126));
				sbSQL.Append(DTPConfig.GetSQLText(127));
				sbSQL.AppendFormat(DTPConfig.GetSQLText(128), lSchemaID);
				
				return GetSchemaBySQL(sbSQL.ToString());				
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return string.Empty;
		}
		

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a schema, version, and type 
		/// and see if it exists in the database.
		/// </summary>
		/// <param name="sSchemaName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="eType">Schema Type</param>
		/// <returns>True or False</returns>
		public override bool DoesSchemaExist(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{
			try
			{
				Profiler.BeginInstrumentation();

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return true;

				StringBuilder sb =  null;
				sb = new StringBuilder();
				if(sb == null)
					return false;

				sb.Append(DTPConfig.GetSQLText(170));
				sb.Append(DTPConfig.GetSQLText(171));
				sb.AppendFormat(DTPConfig.GetSQLText(172), sSchemaName, sVersion, Convert.ToInt32(eType));

				int iResult = 0;
				iResult = Convert.ToInt32(oDB.ExecuteScalar(sb));
				if(iResult > 0)
					return true;

				return false;
			}
			catch(SqlException e)
			{
				LogSQLException(e);			
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		
			return false;
		}


		/// <summary>
		/// This method will extract the XSD schema from the Packet's encasulated Packet and insert or update in into the database.
		/// </summary>
		/// <param name="sName">Schema Name</param>
		/// <param name="sDescription"> Description (if blank defaults to '(Empty)'</param>
		/// <param name="oSchema">Packet encasulating the Schema.</param>
		/// <returns>true if successful; false if an error occurred.</returns>
		public override bool DBStoreSchema(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType, string sDescription, DataSet oSchema)
		{
			try
			{			
				Profiler.BeginInstrumentation();

				//	Check argument
				if(sSchemaName.Length == 0)
					return false;

				//	Check argument
				if(eType == DTPConstants.SCHEMA_TYPE.UNKNOWN)
					return false;
				
				//	Check argument
				if(sVersion.Length == 0)
					return false;

				//	Check argument
				if(oSchema == null)
					return false;
				
				//	Check argument
				string sDescription2 = sDescription;
				if(sDescription.Length == 0)
					sDescription2 = "Default Description";
				
				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData("sp_DNPatternsUpdateSchema", CommandType.StoredProcedure);
				if(oDB == null)
					return false;

				//	Bind input stored procedure parameter
				SqlParameter oInParm1 = null;
				
				oInParm1 = (SqlParameter) oDB.AddDataParameter("@Name", SqlDbType.VarChar, 64, ParameterDirection.Input);
				if(oInParm1 == null)
					return false;
				oInParm1.Value = sSchemaName;

				SqlParameter oInParm2 = null;
				oInParm2 = (SqlParameter) oDB.AddDataParameter("@Version", SqlDbType.VarChar, 64, ParameterDirection.Input);
				if(oInParm2 == null)
					return false;
				oInParm2.Value = sVersion;

				SqlParameter oInParm3 = null;
				oInParm3 = (SqlParameter) oDB.AddDataParameter("@SchemaType", SqlDbType.BigInt, ParameterDirection.Input);
				if(oInParm3 == null)
					return false;
				oInParm3.Value = (long) eType;

				//	Bind input stored procedure parameter
				SqlParameter oInParm4 = null;
				oInParm4 = (SqlParameter) oDB.AddDataParameter("@Description", SqlDbType.VarChar, 255, ParameterDirection.Input);
				if(oInParm4 == null)
					return false;
				oInParm4.Value = sDescription2;								

				//	Bind input stored procedure parameter
				SqlParameter oInParm5 = null;
				oInParm5 = (SqlParameter) oDB.AddDataParameter("@XMLData", SqlDbType.Text, ParameterDirection.Input);
				if(oInParm5 == null)
					return false;
				string sXML = string.Empty;
				
				sXML = oDB.WriteXml(oSchema, XmlWriteMode.WriteSchema);
				if(sXML.Length == 0)
					return false;
				oInParm5.Value = sXML;								
				
				//	Bind output stored procedure parameter (holds T-SQL RETURN(x) value)
				SqlParameter oOutParm = null;
				oOutParm = (SqlParameter) oDB.AddDataParameter("@RowCount", SqlDbType.Int, ParameterDirection.ReturnValue);
				if(oOutParm == null)
					return false;
								
				//	Execute the stored procedure
				int iRowCount = 0;
				iRowCount = oDB.ExecuteNonQuery();				

				//	Get the result of the stored procedure
				int iCount = 0;
				iCount = Convert.ToInt32(oDB.GetDataParameterValue("@RowCount"));
				if(iCount == 1)				
					return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}
		

		/// <summary>
		/// This method will accept a SQL command to execute. The results will be used to hydrate a Packet with the 
		/// data stored in the database for a transaction. This method does the actual RDBMS I/O.
		/// </summary>
		/// <param name="sSQL">SQL command to execute</param>
		/// <returns>Packet</returns>
		public override Packet DBGetTransactionBySQL(string sSQL)
		{
			try
			{
				Profiler.BeginInstrumentation();

				if(sSQL.Length == 0)
					return null;

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return null;
				
				int iResult = 0;
				iResult = oDB.ExecuteFillQuery(sSQL);
				if(iResult == 0)
					return null;

				//	See if the DataTable Table exists
				if(oDB.Contains(DTPConstants.DEFAULT_TABLENAME) == false)
					return null;

				long lSchemaID = 0;
				lSchemaID = (long) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_SCHEMA_ID_COLUMN_NAME];				
				string sXMLSchema = string.Empty;
				sXMLSchema = GetSchemaAsString(lSchemaID);
				if(sXMLSchema.Length == 0)
					return null;

				PolyModelData oDB2 = ModelFactory.CreatePolyModelData();
				if(oDB2 == null)
					return null;

				oDB2.ReadXmlSchema(sXMLSchema);
								
				//	Extract XML for transaction from the DataTable
				string sXML = string.Empty;
				byte[] byXML = null;
				byXML = (byte[]) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_XML_COLUMN_NAME];
				if(byXML != null)
					sXML = Encoding.ASCII.GetString(byXML);
				if(sXML.Length == 0)
					return null;

				oDB2.ReadXml(sXML);

				Packet oPacket = new Packet();
				oPacket.RawData = oDB2.Data;

				//	Extract transaction id and the schema id
				oPacket.TransactionID = (long) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_TRANS_ID_NAME];
				oPacket.ProductSchemaID = lSchemaID;

				//	Set the schema identity id
				oPacket.ProductIdentitySchemaID = (long) GetCurrentLookupSchemaID(oPacket.ProductSchemaID);

				return oPacket;
			}		
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}		
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This method will do the acutal RDBMS I/O to read and populated a Packet with a transaction stored in the database.
		/// </summary>
		/// <param name="lTransID"></param>
		/// <returns></returns>
		public override Packet DBGetTransactionRecord(long lTransID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lTransID == 0)
					return null;

				//	Make sure the transaction exists before we attempt to load it from the database.
				if(DoesTransactionExist(lTransID) == false)
					return null;

				StringBuilder sb = null;
				sb = new StringBuilder();
				
				//	Define query to execute
				sb.Append("execute sp_executesql N'SELECT * FROM dbo.DATA WHERE TRANS_ID = @TransID', ");
				sb.AppendFormat("N'@TransID BIGINT', @TransID = {0}", lTransID);

				return DBGetTransactionBySQL(sb.ToString());
			}		
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This method will accept an XSD product schema that it will use to find all the transaction associated with an 
		/// the schema in the database.  Each transaction will be processed one at a time and appended to a Packet 
		/// collection to be returned to the calling function.
		/// </summary>
		/// <param name="lSchemaID">XSD Product Schema</param>
		/// <returns>PacketCollection</returns>
		public override PacketCollection GetTransactions(long lSchemaID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lSchemaID == 0)
					return null;

				StringBuilder sb = null;
				sb = new StringBuilder();
				
				//	Define query to execute
				sb.Append("sp_executesql N'SELECT * FROM dbo.DATA WHERE SCHEMA_ID = @SchemaID ORDER BY TRANS_ID', ");
				sb.AppendFormat("N'@SchemaID BIGINT', @SchemaID = {0}", lSchemaID);

				return DBGetTransactionsBySQL(sb.ToString());
			}		
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This method will check and see if a transaction exists in the database.
		/// </summary>
		/// <param name="lTransID">Transaction ID</param>
		/// <returns>True or False</returns>
		public override bool DoesTransactionExist(long lTransID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return true;

				string sSQL = string.Empty;
				sSQL = string.Format("execute sp_executesql N'SELECT COUNT(*) FROM dbo.DATA WHERE TRANS_ID = @TransID', N'@TransID BIGINT', @TransID = {0}", lTransID);

				int iResult = 0;
				iResult = Convert.ToInt32(oDB.ExecuteScalar(sSQL));
				if(iResult > 0)
					return true;

				return false;
			}
			catch(SqlException e)
			{
				LogSQLException(e);			
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		
			return false;
		}


		/// <summary>
		/// This method will accept an dynamic SQL command that it will use to find all the transaction in the database 
		/// that meet the search criteria of the SQL command.  Each transaction will be processed one at a time and 
		/// appended to a Packet collection to be returned to the calling function.
		/// </summary>
		/// <param name="sSQL">SQL to execute</param>
		/// <returns>PacketCollection</returns>
		public override PacketCollection DBGetTransactionsBySQL(string sSQL)
		{			
			try
			{
				Profiler.BeginInstrumentation();

				if(sSQL.Length == 0)
					return null;

				//	Create a collection to hold the results
				PacketCollection oOut = null;
				oOut = new PacketCollection();
				
				//	Execute the query and call Fill()
				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return null;

				int iResult = 0;
				iResult = oDB.ExecuteFillQuery(sSQL);

				//	See if the DataTable Table exists
				if(oDB.Contains(DTPConstants.DEFAULT_TABLENAME) == false)
					return null;

				//	Get the numer of rows in the PolyModelData encasulated DataSet 
				DataTable oTable = null;
				oTable = oDB[DTPConstants.DEFAULT_TABLENAME];
				int iRowCount = 0;
				iRowCount = oTable.Rows.Count;

				long lOldSchemaID = 0;
				long lIdentitySchema = 0;

				string sXMLSchema = string.Empty;

				//	Now lets process those rows in the Table DataTable one at a time.  Each row represents an individual
				//	transaction.
				for(int i = 0; i < iRowCount; i++)
				{
					//	Get the schema associated with the specific row.
					long lNewSchemaID = 0;
					lNewSchemaID = (long) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_SCHEMA_ID_COLUMN_NAME, i];	
					
					//	Are dealing with the same schema,or same batch?
					if(lOldSchemaID != lNewSchemaID)
					{			
						//	No, lets load the new schema from the database
						sXMLSchema = GetSchemaAsString(lNewSchemaID);
						if(sXMLSchema.Length == 0)
							return null;

						//	Save the schema ID for the new schema
						lOldSchemaID = lNewSchemaID;
						
						//	Save the identity schema id
						lIdentitySchema = (long) GetCurrentLookupSchemaID(lOldSchemaID);
					}

					PolyModelData oDB2 = ModelFactory.CreatePolyModelData();
					if(oDB2 == null)
						return null;

					oDB2.ReadXmlSchema(sXMLSchema);
									
					//	Extract XML for transaction from the DataTable
					string sXML = string.Empty;
					byte[] byXML = null;
					byXML = (byte[]) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_XML_COLUMN_NAME, i];
					if(byXML != null)
						sXML = Encoding.ASCII.GetString(byXML);
					if(sXML.Length == 0)
						return null;

					oDB2.ReadXml(sXML);

					//	Hydrate a packet from the DataSet
					PacketTranslator oTranslator = null;
					oTranslator =  new PacketTranslator();
					
					Packet oPacket = null;
					oPacket = oTranslator.Translate(oDB2.Data);
					if(oPacket == null)
						return null;				

					//	Extract transaction id and the schema id
					oPacket.TransactionID = (long) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_TRANS_ID_NAME, i];
					oPacket.ProductSchemaID = lOldSchemaID;

					//	Set the schema identity id
					oPacket.ProductIdentitySchemaID = lIdentitySchema;

					oOut.Add(oPacket);
				}

				return oOut;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}		
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
			
			return null;
		}
						

		/// <summary>
		/// This method will write a transaction to the database.  This method also included a retry loop that is to attempt multiple 
		/// writer to the DATA_LOG table. 
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <returns>True or false</returns>
		public override bool DBStoreTransactionRecord(Packet oTrans)
		{			
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(oTrans == null)
					return false;

				int iNumTries = 0;

				bool bResult = false;

				StringBuilder sb = null;

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Write the database log record to the DATA_LOG table
					bResult = DBStoreTransactionRecord(oTrans, "sp_DNPatternsUpdateDataLog");
					if(bResult)
					{
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							sb.AppendFormat("(Empty)", iNumTries, oTrans.TransactionID);				
							Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
						}
									
						//	Write the database record to the DATA table
						return DBStoreTransactionRecord(oTrans, "sp_DNPatternsUpdateData");
					}

					sb = null;
					sb = new StringBuilder();
					sb.AppendFormat("Attempt {0} - Failed to save transaction '{1}'.", iNumTries, oTrans.TransactionID);				
					Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
							
					iNumTries++;
				}	

				sb = null;
				sb = new StringBuilder();
				sb.AppendFormat("execute sp_executesql N'SELECT COUNT(*) FROM dbo.COUNTER WHERE NAME = @Name', N'@Name VARCHAR(64)', @Name = '{0}'", oTrans.TransactionID);				
				Trace.WriteLineIf(Config.TraceError, sb.ToString());
				
				return false;
			}
			catch(Exception e)
			{				
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}


		/// <summary>
		/// This method will write a transaction to the database.  The stored procedure it will use is passed in as 
		/// an argument.
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <param name="sStoredProcedureName">Stored Procedure Name</param>
		/// <returns>True or False</returns>
		public override bool DBStoreTransactionRecord(Packet oTrans, string sStoredProcedureName)
		{
			try
			{		
				Profiler.BeginInstrumentation();

				//	Check argument
				if(oTrans == null)
					return false;		
				
				//	Check argument
				if(sStoredProcedureName.Length == 0)
					return false;
				
				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData(sStoredProcedureName, CommandType.StoredProcedure);
				if(oDB == null)
					return false;

				//	Bind input stored procedure parameter
				SqlParameter oInParm1 = null;
				oInParm1 = (SqlParameter) oDB.AddDataParameter("@TransID", SqlDbType.BigInt, ParameterDirection.Input);
				if(oInParm1 == null)
					return false;
				oInParm1.Value = oTrans.TransactionID;

				//	Bind input stored procedure parameter
				SqlParameter oInParm2 = null;
				oInParm2 = (SqlParameter) oDB.AddDataParameter("@SchemaID", SqlDbType.BigInt, ParameterDirection.Input);
				if(oInParm2 == null)
					return false;
				oInParm2.Value = oTrans.ProductSchemaID;								

				//	Bind input stored procedure parameter
				SqlParameter oInParm3 = null;
				oInParm3 = (SqlParameter) oDB.AddDataParameter("@XMLData", SqlDbType.Image, ParameterDirection.Input);
				if(oInParm3 == null)
					return false;
				string sXML = string.Empty;
				sXML = oDB.WriteXml(oTrans.RawData, XmlWriteMode.IgnoreSchema);				
				oInParm3.Value = Encoding.ASCII.GetBytes(sXML);
				
				//	Bind output stored procedure parameter (holds T-SQL RETURN(x) value)
				SqlParameter oOutParm = null;
				oOutParm = (SqlParameter) oDB.AddDataParameter("@RowCount", SqlDbType.Int, ParameterDirection.ReturnValue);
				if(oOutParm == null)
					return false;
								
				//	Execute the stored procedure
				int iRowCount = 0;
				iRowCount = oDB.ExecuteNonQuery();				

				//	Get the result of the stored procedure
				int iCount = 0;
				iCount = Convert.ToInt32(oDB.GetDataParameterValue("@RowCount"));
				if(iCount == 1)				
					return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}


		#region Counter Methods

		/// <summary>
		/// This method will return the current value of a counter.
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <returns>Current counter value or -1 if an error occurs</returns>
		public override long GetCounterValue(string sCounterName)	
		{
			long lResult = DTPConstants.DB_ERROR;

			try
			{			
				Profiler.BeginInstrumentation();

				//	Make sure the counter exists first
				if(DoesCounterExist(sCounterName) == false)
					return DTPConstants.DB_ERROR;

				//	Define dynamic SQL to execute
				StringBuilder sbSQL = null;
				sbSQL =  new StringBuilder();
				if(sbSQL == null)
					return lResult;

				sbSQL.Append(DTPConfig.GetSQLText(145));
				sbSQL.Append(DTPConfig.GetSQLText(146));
				sbSQL.Append(DTPConfig.GetSQLText(147));
				sbSQL.AppendFormat(DTPConfig.GetSQLText(148), sCounterName);

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return DTPConstants.DB_ERROR;
				
				//	Execute the query
				lResult = Convert.ToInt32(oDB.ExecuteScalar(sbSQL));
				return lResult;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return DTPConstants.DB_ERROR;
		}
		

		/// <summary>
		/// This method will insert or update a counter with the value passed in lValue.  If the counter does not exists, 
		/// it is created.  A stored called 'sp_PMUpdCounter' is used to ensure data concurrency is enforced.
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <param name="lValue">New value of the counter</param>
		/// <returns>true if successful; false if an error occurred</returns>
		public override bool StoreCounterValue(string sCounterName, long lValue)
		{
			try
			{	
				Profiler.BeginInstrumentation();

				//	Check argument
				if(sCounterName.Length == 0)
					return false;
				
				//	Check argument
				if(lValue == 0)
					return false;

				//	Make sure we are not attempting to update TRANS_ID from here since TRANS_ID can only be updated by calling GenTransID
				if(sCounterName.Equals(DTPConstants.DEFAULT_TRANS_ID_NAME) == true)
					return false;

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData("sp_DNPatternsUpdateCounter", CommandType.StoredProcedure);
				if(oDB == null)
					return false;

				//	Bind input stored procedure parameter
				SqlParameter oInParm1 = null;
				oInParm1 = (SqlParameter) oDB.AddDataParameter("@Name", SqlDbType.VarChar, 64,ParameterDirection.Input);
				if(oInParm1 == null)
					return false;
				oInParm1.Value = sCounterName;

				//	Bind input stored procedure parameter
				SqlParameter oInParm2 = null;
				oInParm2 = (SqlParameter) oDB.AddDataParameter("@Value", SqlDbType.BigInt);
				if(oInParm2 == null)
					return false;
				oInParm2.Value = lValue;						
				
				//	Bind output stored procedure parameter (holds T-SQL RETURN(x) value)
				SqlParameter oOutParm = null;
				oOutParm = (SqlParameter) oDB.AddDataParameter("@RowCount", SqlDbType.Int, ParameterDirection.ReturnValue);
								
				//	Execute the stored procedure
				int iRowCount = 0;
				iRowCount = oDB.ExecuteNonQuery();				

				//	Get the result of the stored procedure
				int iCount = 0;
				iCount = Convert.ToInt32(oDB.GetDataParameterValue("@RowCount"));
				if(iCount == 1)				
					return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);			
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;			
		}		


		/// <summary>
		/// This method will automatically create and/or update sCounterName by incrementing the
		/// current value by 1.  If the counter does not exists, it will be created and the default value
		/// passed in will be used as the initialization value for the counter.  
		/// 
		/// Do not call this method if you need a Transaction Identifier aka TRANS_ID - use GenerateTransactionID() instead.
		/// 
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <param name="lDefaultValue">Default value of the counter.</param>
		/// <returns>Next counter value or -1 if an error occurs</returns>
		public override long GetNextCounterValue(string sCounterName, long lDefaultValue)
		{
			long lResult = DTPConstants.DB_ERROR;

			try
			{	
				Profiler.BeginInstrumentation();

				//	See if we need to change from -1 to 1 (overloads pass in -1)
				long lDefaultValue2 = lDefaultValue;
				if(lDefaultValue2 == DTPConstants.DB_ERROR)
					lDefaultValue2 = DTPConstants.DEFAULT_COUNTER_VALUE;
				
				//	Define dynamic SQL to execute
				StringBuilder sbSQL = null;
				sbSQL = new System.Text.StringBuilder();
				if(sbSQL == null)
					return lResult;

				sbSQL.Append("exec sp_DNPatternsIncCounter ");
				sbSQL.AppendFormat("'{0}', ",  sCounterName);
				sbSQL.AppendFormat("{0}", lDefaultValue2.ToString());

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return DTPConstants.DB_ERROR;

				//	Execute the query				
				lResult = Convert.ToInt64(oDB.ExecuteScalar(sbSQL));
				return lResult;
			}
			catch(SqlException e)
			{
				LogSQLException(e);			
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return DTPConstants.DB_ERROR;			
		}


		/// <summary>
		/// This method will check and see if a specific counter exists in the database.
		/// </summary>
		/// <param name="sCounterName">Counter Name</param>
		/// <returns>True or False</returns>
		public override bool DoesCounterExist(string sCounterName)
		{
			try
			{
				Profiler.BeginInstrumentation();

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return true;

				string sSQL = string.Empty;
				sSQL = string.Format(DTPConfig.GetSQLText(168), sCounterName);

				int iResult = 0;
				iResult = Convert.ToInt32(oDB.ExecuteScalar(sSQL));
				if(iResult > 0)
					return true;

				return false;
			}
			catch(SqlException e)
			{
				LogSQLException(e);			
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		
			return false;
		}


		#endregion
		
		/// <summary>
		/// This method will get the current database date and time.  Uses GETDATE().  If an error occurs, 
		/// the date will return formatted as January 31, 1899 12:59:59 AM based on the LOCAL settings of the  server.
		/// </summary>
		/// <returns>DateTime</returns>
		public override DateTime GetCurrentDBDate()
		{
			DateTime dtResult = new DateTime(1899, 01, 31, 12, 59, 59, Calendar.CurrentEra);

			try
			{	
				Profiler.BeginInstrumentation();

				//	Define dynamic SQL to execute
				string sSQL = string.Empty;
				sSQL = "execute sp_executesql N'SELECT GETDATE() AS DBDATE'";
								
				//	Execute the query				
				PolyModelData oDB = null;

				oDB = ModelFactory.CreatePolyModelData();
				if(oDB != null)
					dtResult = Convert.ToDateTime(oDB.ExecuteScalar(sSQL));
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return dtResult;
		}


		#region Key Builder Methods

		/// <summary>
		/// This method will remove all lookup keys from the database that are associated to the passed in Packet.  
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <returns>True or False</returns>
		public override bool RemoveAllKeys(Packet oTrans)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(oTrans == null)
					return false;
				
				//	Check transaction id
				if(oTrans.TransactionID == 0)
					return false;

				StringBuilder sbSQL = null;
				sbSQL = new StringBuilder();
				if(sbSQL == null)
					return false;

				//	Define the query to execute
				sbSQL.Append("execute sp_executesql N'DELETE FROM dbo.DATA_IDX WHERE TRANS_ID = @TransID', ");
				sbSQL.AppendFormat("N'@TransID BIGINT', @TransID = {0}", oTrans.TransactionID);

				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return false;

				//	Execute the query
				int iRowCount = 0;
				iRowCount = Convert.ToInt32(oDB.ExecuteNonQuery(sbSQL));

				return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}


		/// <summary>
		/// This method will insert all the lookup keys into the database that are associated to the passed in Packet.  
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <returns>True or False</returns>
		public override bool StoreKeys(Packet oTrans)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(oTrans == null)
					return false;

				//	Delete all existing keys
				bool bRemoveKeys = false;
				bRemoveKeys = RemoveAllKeys(oTrans);

				string sXMLSchema = null;
				sXMLSchema = GetSchemaAsString(oTrans.ProductIdentitySchemaID);
				if(sXMLSchema.Length == 0)
					return false;

				PolyModelData oDB = ModelFactory.CreatePolyModelData();
				oDB.ReadXml(sXMLSchema, XmlReadMode.ReadSchema);

				if(oDB.Contains("LOOKUP") == false)
					return false;

				int iRowCount = 0;
				iRowCount = oDB.Data.Tables[DTPConstants.LOOKUP_DATATABLENAME].Rows.Count;
				
				for(int i = 0; i < iRowCount; i++)
				{
					string sTable = string.Empty;
					sTable = Convert.ToString(oDB[DTPConstants.LOOKUP_DATATABLENAME, DTPConstants.LOOKUP_DATACOLUMN_TABLE, i]);
					if(sTable.Length == 0)
						continue;
					
					string sColumn = string.Empty;
					sColumn = Convert.ToString(oDB[DTPConstants.LOOKUP_DATATABLENAME, DTPConstants.LOOKUP_DATACOLUMN_COLUMN, i]);
					if(sColumn.Length == 0)
						continue;

					string sValue = string.Empty;
					// use this line when NOT using the packet translator
					sValue = oTrans.RawData.Tables[sTable].Rows[0][sColumn].ToString();
					// when using Packet Translator use the following line
					//sValue = oTrans[sTable, sColumn];

					if(sValue.Length == 0)
						continue;

					bool bResult = false;
					bResult = StoreKey(oTrans.TransactionID, sColumn, sValue);
				}	
				
				return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}


		/// <summary>
		/// This method with insert a single lookup key into the database that is accociated to the passed in Packet.
		/// </summary>
		/// <param name="lTransID">Transaction ID</param>
		/// <param name="sKeyName">Look Up Key Name</param>
		/// <param name="sValue">Value of Look Up Key</param>
		/// <returns>True or False</returns>
		public override bool StoreKey(long lTransID, string sKeyName, string sValue)
		{
			try
			{	
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lTransID == 0)
					return false;

				//	Check argument
				if(sKeyName.Length == 0)
					return false;
				
				//	Check argument
				if(sValue.Length == 0)
					return false;
				
				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData("sp_DNPatternsUpdateDataIdx", CommandType.StoredProcedure);
				if(oDB == null)
					return false;

				//	Bind input stored procedure parameter
				SqlParameter oInParm1 = null;
				oInParm1 = (SqlParameter) oDB.AddDataParameter("@TransID", SqlDbType.BigInt, ParameterDirection.Input);
				if(oInParm1 == null)
					return false;
				oInParm1.Value = lTransID;

				//	Bind input stored procedure parameter
				SqlParameter oInParm2 = null;
				oInParm2 = (SqlParameter) oDB.AddDataParameter("@Name", SqlDbType.VarChar, 64, ParameterDirection.Input);
				if(oInParm2 == null)
					return false;
				oInParm2.Value = sKeyName;								

				//	Bind input stored procedure parameter
				SqlParameter oInParm3 = null;
				oInParm3 = (SqlParameter) oDB.AddDataParameter("@Value", SqlDbType.VarChar, 255, ParameterDirection.Input);
				if(oInParm3 == null)
					return false;
				oInParm3.Value = sValue;
				
				//	Bind output stored procedure parameter (holds T-SQL RETURN(x) value)
				SqlParameter oOutParm = null;
				oOutParm = (SqlParameter) oDB.AddDataParameter("@RowCount", SqlDbType.Int, ParameterDirection.ReturnValue);
				if(oOutParm == null)
					return false;
								
				//	Execute the stored procedure
				int iRowCount = 0;
				iRowCount = oDB.ExecuteNonQuery();				

				//	Get the result of the stored procedure
				int iCount = 0;
				iCount = Convert.ToInt32(oDB.GetDataParameterValue("@RowCount"));
				if(iCount == 1)				
					return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(Exception e)
			{				
				Trace.WriteLine(e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}		
	

		#endregion
	}
}
