using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Globalization;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Diagnostics;
using DotNetPatterns.Common;

namespace DataTierPatterns.Data
{
	public abstract class PolyModel
	{
		public PolyModel()
		{
			m_oProfiler = new LazyInstrumentation();			
		}

		protected LazyInstrumentation m_oProfiler = null;

		public LazyInstrumentation Profiler
		{
			get	{	return m_oProfiler;	}
		}

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a product and 
		/// return the ID of the current XSD Product Schema.  
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <returns>XSD Product Schema ID or DTPConstants.DB_ERROR if an error occurs</returns>
		public abstract long GetCurrentSchemaIDForProductSchema(string sProductName);

		/// <summary>
		/// This method will take the name of a product schema and return a string populated with the current XSD Schema for a product.
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <returns>Packet hydrated from an XSD Schema for a DataSet.</returns>
		public virtual Packet GetCurrentProductSchema(string sProductName)
		{			
			try
			{
				Profiler.BeginInstrumentation();

				return GetCurrentSchema(sProductName, DTPConstants.SCHEMA_TYPE.PRODUCT);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}
		
		
		/// <summary>
		/// This method will take the name of a product schema and return a string populated with the current identity XSD Schema for a product.
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <returns>Packet hydrated from an identity XSD Schema for a DataSet.</returns>
		public virtual Packet GetCurrentLookupSchema(string sProductName)
		{	
			try
			{
				Profiler.BeginInstrumentation();

				return GetCurrentSchema(sProductName, DTPConstants.SCHEMA_TYPE.IDENTITY);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}


		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a product schema and 
		/// return the ID of the current identity schema associated with the product schema.  
		/// </summary>
		/// <param name="lProductSchemaID">Product Name</param>
		/// <returns>Identity Schema ID or DTPConstants.DB_ERROR if an error occurs</returns>
		public abstract long GetCurrentLookupSchemaID(long lProductSchemaID);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a product schema and 
		/// schema type and return a string that is populated with the current XSD Schema that matches the search criteria.
		/// </summary>
		/// <param name="sProductName">Product Name</param>
		/// <param name="eType">Type of Schema</param>
		/// <returns>Packet hydrated from an XSD Schema for a DataSet.</returns>		
		public abstract Packet GetCurrentSchema(string sProductName, DTPConstants.SCHEMA_TYPE eType);
	
		/// <summary>
		/// This method will take the name of a schema, version, and type and return a Packet hydrate with 
		/// an XSD Schema for a DataSet.
		/// </summary>
		/// <param name="sSchemaName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="iType">SchemaType</param>
		/// <returns>Packet or null if an error occurred.</returns>
		public virtual Packet GetSchema(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{				
			try
			{
				Profiler.BeginInstrumentation();

				// Check argument
				if(sSchemaName.Length == 0)
					return null;
				
				// Check argument
				if(sVersion.Length == 0)
					return null;
				
				// Check argument
				if(eType == DTPConstants.SCHEMA_TYPE.UNKNOWN)
					return null;

				string sXMlSchema = string.Empty;
				sXMlSchema = GetSchemaAsString(sSchemaName, sVersion, eType);
				if(sXMlSchema.Length == 0)
					return null;
				
				//	Return the hydrated schema
				return XML2Packet(sXMlSchema);			
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}		

		
		/// <summary>
		/// This method will take the ID of a schema and return a Packet Hydrate with an XSD Schema of a DataSet.
		/// </summary>
		/// <param name="lSchemaID">Schema ID</param>
		/// <returns>Packet or null if an error occurred.</returns>
		public virtual Packet GetSchema(long lSchemaID)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lSchemaID == 0)
					return null;

				string sXMlSchema = string.Empty;
				sXMlSchema = GetSchemaAsString(lSchemaID);
				if(sXMlSchema.Length == 0)
					return null;

				//	Return the hydrated schema
				return XML2Packet(sXMlSchema);
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;		
		}
				
		
		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a schema, version, and type 
		/// and return a string populated with the XSD Schema for a DataSet.
		/// </summary>
		/// <param name="sSchemaName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="iType">Schema Type</param>
		/// <returns>string populated with an XSD Schema of a DataSet; or a empty string if an error occurred.</returns>
		public abstract string GetSchemaAsString(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType);	

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the ID of a SCHEMA 
		/// and return a string populated with the XSD Schema of a DataSet.
		/// </summary>
		/// <param name="lSchemaID">Schema ID</param>
		/// <returns>string populated with an XSD Schema of a DataSet ; or a empty string if an error occurred.</returns>
		public abstract string GetSchemaAsString(long lSchemaID);		
		
		public virtual string GetSchemaBySQL(string sSQL)
		{
			string sVersionFiller = "filler";
			return GetSchemaBySQL(sSQL, out sVersionFiller);
		}

		/// <summary>
		/// This is a helper method that will go to the databse and return a Packet object that represents a Product's
		/// XSD schema.  This method is called by both the GetSchema() overloads that are defined in the interface.
		/// </summary>
		/// <param name="sSQL">SQL query to execute against the SCHEMA table</param>
		/// <returns>string populated with an XSD Schema of a DataSet or null if an error occurred.</returns>
		public virtual string GetSchemaBySQL(string sSQL, out string sVersion)
		{
			sVersion = "0";
				
			try
			{
				Profiler.BeginInstrumentation();

				if(sSQL.Length == 0)
					return string.Empty;

				PolyModelData oDB =  null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return string.Empty;

				int iRowCount = 0;
				iRowCount = oDB.ExecuteFillQuery(sSQL);

				if(oDB.Contains(DTPConstants.DEFAULT_TABLENAME) == true)
				{
					sVersion = (string) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_VERSION_COLUMN_NAME];
					return (string) oDB[DTPConstants.DEFAULT_TABLENAME, DTPConstants.DEFAULT_XML_COLUMN_NAME];
				}
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(OleDbException e)
			{
				LogOleDbException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return string.Empty;
		}


		/// <summary>
		/// This method will store the XSD Schema encasulated in the Packet in the database.  Retry-logic is implemented.
		/// </summary>
		/// <param name="sName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="iType">Schema Type</param>
		/// <param name="sDescription">Schema Description</param>
		/// <param name="oSchema">Packet</param>
		/// <returns>True or False</returns>
		public virtual bool StoreSchema(string sName, string sVersion, DTPConstants.SCHEMA_TYPE eType, string sDescription, DataSet oSchema)
		{	
			bool bResult = false;

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(eType == DTPConstants.SCHEMA_TYPE.UNKNOWN)
					throw new BaseException(this, 1111, "Unknown schema type passed", false);	

				int iNumTries = 0;

				StringBuilder sb = null;

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Write the schema to the database
					bResult = DBStoreSchema(sName, sVersion, eType, sDescription, oSchema);
					if(bResult)
					{
						//	See if we are in a retry loop
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							if(sb != null)
							{						
								sb.AppendFormat(DTPConfig.GetCommonText(100), iNumTries, sName, sVersion, Convert.ToInt32(eType), sDescription);
								Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
							}
						}

						return true;						
					}
					
					sb = null;
					sb = new StringBuilder();
					if(sb != null)
					{
						sb.AppendFormat(DTPConfig.GetCommonText(101), iNumTries, sName, sVersion, Convert.ToInt32(eType), sDescription);			
						Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
					}
						
					iNumTries++;
				}	

				sb = null;
				sb = new StringBuilder();
				if(sb != null)
				{
					sb.AppendFormat(DTPConfig.GetCommonText(102), sName, sVersion, Convert.ToInt32(eType), sDescription);		
					Trace.WriteLineIf(Config.TraceError, sb.ToString());
				}

				return false;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}
		

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will take the name of a schema, version, and type 
		/// and see if it exists in the database.
		/// </summary>
		/// <param name="sSchemaName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="eType">Schema Type</param>
		/// <returns>True or False</returns>
		public abstract bool DoesSchemaExist(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will do the acutal RDBMS I/O to store 
		/// the XSD Schema encasulated in the Packet in the database.
		/// </summary>
		/// <param name="sName">Schema Name</param>
		/// <param name="sVersion">Schema Version</param>
		/// <param name="iType">Schema Type</param>
		/// <param name="sDescription">Schema Description</param>
		/// <param name="oSchema">Packet</param>
		/// <returns>True or False</returns>
		public abstract bool DBStoreSchema(string sName, string sVersion, DTPConstants.SCHEMA_TYPE eType, string sDescription, DataSet oSchema);		

		/// <summary>
		/// This method will accept a Packet and read the TransactionID property that is will use to go to the database and 
		/// return a new Packet populated with the transaction information for the TransactionID.
		/// </summary>
		/// <param name="oPacket">Packet</param>
		/// <returns>Packet</returns>
		public virtual Packet GetTransaction(Packet oPacket)
		{
			try
			{
				Profiler.BeginInstrumentation();

				if(oPacket != null)
					return GetTransaction(oPacket.TransactionID);

				return null;
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}
		

		/// <summary>
		/// The method will take the ID of a transaction and return a Packet hydrate from the diffgram stored in the
		/// database.  This method uses retry-logic.
		/// </summary>
		/// <param name="lTransID">Transaction ID</param>
		/// <returns>Packet</returns>
		public virtual Packet GetTransaction(long lTransID)
		{		
			StringBuilder sb = null;

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(lTransID == 0)
					return null;

				Packet oOut = null;

				int iNumTries = 0;			

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Read the transaction from the database
					oOut = DBGetTransactionRecord(lTransID);
					if(oOut != null)
					{
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							if(sb != null)
							{
								sb.AppendFormat(DTPConfig.GetCommonText(103), iNumTries, lTransID);				
								Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
							}
						}
						
						return oOut;
					}
					
					sb = null;
					sb = new StringBuilder();
					if(sb != null)
					{
						sb.AppendFormat(DTPConfig.GetCommonText(104), iNumTries, lTransID);			
						Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
					}
						
					iNumTries++;
				}	

				sb = null;
				sb = new StringBuilder();
				if(sb != null)
				{
					sb.AppendFormat(DTPConfig.GetCommonText(105), lTransID);		
					Trace.WriteLineIf(Config.TraceError, sb.ToString());
				}

				return null;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This abstract method that is to be implemented in the derived class will 
		/// do the acutal RDBMS I/O to read and populated a Packet with a transaction stored in the database.
		/// </summary>
		/// <param name="lTransID"></param>
		/// <returns></returns>
		public abstract Packet DBGetTransactionRecord(long lTransID);

		/// <summary>
		/// This method will return a transaction from the database that matches the collection of passed in 
		/// where criteria.
		/// </summary>
		/// <param name="colWhere">Collection of where criteria</param>
		/// <returns>Packet</returns>
		public virtual Packet GetTransaction(PolyModelWhereCollection colWhere)
		{
			StringBuilder sb = null;

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(colWhere == null)
					return null;

				Packet oOut = null;

				int iNumTries = 0;			

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Read the transaction from the database
					oOut = DBGetTransactionBySQL(colWhere.GetSQL());
					if(oOut != null)
					{
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							if(sb != null)
							{
								sb.AppendFormat(DTPConfig.GetCommonText(103), iNumTries);
								Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
							}
						}
						
						return oOut;
					}
					
					sb = null;
					sb = new StringBuilder();
					if(sb != null)
					{
						sb.AppendFormat(DTPConfig.GetCommonText(104), iNumTries);			
						Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
					}
						
					iNumTries++;
				}	

				sb = null;
				sb = new StringBuilder();
				if(sb != null)
				{
					sb.Append(DTPConfig.GetCommonText(105));		
					Trace.WriteLineIf(Config.TraceError, sb.ToString());
				}

				return null;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}		
		

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will accept a SQL command to execute.  
		/// The results will be used to hydrate a Packet with the data stored in the database for a transaction.  
		/// This method does the actual RDBMS I/O.
		/// </summary>
		/// <param name="sSQL">SQL command to execute</param>
		/// <returns>Packet</returns>
		public abstract Packet DBGetTransactionBySQL(string sSQL);

		/// <summary>
		/// This method will accept an array of Transaction ID's and process them one at a time by loading the 
		/// specific transaction from the database and appending it to a PacketCollection that will be returned to
		/// the calling functions.
		/// </summary>
		/// <param name="arlTransIDs">An array of Transaction ID's</param>
		/// <returns>PacketCollection</returns>
		public virtual PacketCollection GetTransactions(long[] arlTransIDs)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Create a collection to hold the results
				PacketCollection oOut = null;
				oOut = new PacketCollection();
				if(oOut == null)
					return null;
				
				//	Get the number of items in the arrauy
				int iTotal = 0;
				iTotal = arlTransIDs.GetUpperBound(0);			
				if(iTotal == -1)
					return oOut;	//	Nothing to process, return an empty array

				iTotal++;	//	Add 1 to the count since the upperbound value is not the same as Count or Size
				
				//	Check and see if there is anything to process
				if(iTotal == 0)
					return oOut;	//	No, lets bail
		
				//	Process each transaction one at a time and add it to the collection
				for(int i = 0; i < iTotal; i++)
				{
					//	Get a transaction id from the collection
					long lTransID = 0;
					lTransID = arlTransIDs[i];
					if(lTransID == 0)
						continue;

					//	Go to the database and load the transaction
					Packet oPacket = null;
					oPacket = GetTransaction(lTransID);
					if(oPacket != null)
						oOut.Add(oPacket);	//	Add the transaction to the collection
				}

				return oOut;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This method accepts a collection of where criteria that is passed to the database to return all the transactions
		/// that match the specific where criteria.  As each transaction is processed, it is appended into a 
		/// PacketCollection that will be returned to the calling functions.
		/// </summary>
		/// <param name="colWhere">Collection of where criteria</param>
		/// <returns>PacketCollection</returns>
		public virtual PacketCollection GetTransactions(PolyModelWhereCollection colWhere)
		{
			StringBuilder sb = null;

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(colWhere == null)
					return null;

				PacketCollection oOut = null;

				int iNumTries = 0;			

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Read the transaction from the database
					oOut = DBGetTransactionsBySQL(colWhere.GetSQL());
					if(oOut != null)
					{
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							if(sb != null)
							{
								sb.AppendFormat(DTPConfig.GetCommonText(106), iNumTries);				
								Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
							}
						}
						
						return oOut;
					}
					
					sb = null;
					sb = new StringBuilder();
					if(sb != null)
					{
						sb.AppendFormat(DTPConfig.GetCommonText(107), iNumTries);			
						Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
					}
						
					iNumTries++;
				}	

				sb = null;
				sb = new StringBuilder();
				if(sb != null)
				{
					sb.Append(DTPConfig.GetCommonText(108));		
					Trace.WriteLineIf(Config.TraceError, sb.ToString());
				}

				return null;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		/// <summary>
		/// This abstract method that is to be implemented in the derived class will accept an XSD product schema that it will
		/// use to find all the transaction associated with an the schema in the database.  Each transaction will be processed one
		/// at a time and appended to a Packet collection to be returned to the calling function.
		/// </summary>
		/// <param name="lSchemaID">XSD Product Schema</param>
		/// <returns>PacketCollection</returns>
		public abstract PacketCollection GetTransactions(long lSchemaID);

		/// <summary>
		/// This method accepts a schema ID and a  collection of where criteria that is passed to the database 
		/// to return all the transactions that match the specific where criteria.  As each transaction is 
		/// processed, it is appended into a PacketCollection that will be returned to the calling functions.
		/// </summary>
		/// <param name="lSchemaID">XSD Product Schema</param>
		/// <param name="colWhere">Collection of where criteria</param>
		/// <returns>PacketCollection</returns>
		public virtual PacketCollection GetTransactions(long lSchemaID, PolyModelWhereCollection colWhere)
		{
			try
			{
				Profiler.BeginInstrumentation();

				return DBGetTransactionsBySQL(colWhere.GetSQL(lSchemaID));
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}


		/// <summary>
		/// This abstract method that is to be implemented in the derived class will accept an dynamic SQL command that it will
		/// use to find all the transaction in the database that meet the search criteria of the SQL command.  Each transaction 
		/// will be processed one at a time and appended to a Packet collection to be returned to the calling function.
		/// </summary>
		/// <param name="sSQL">SQL to execute</param>
		/// <returns>PacketCollection</returns>
		public abstract PacketCollection DBGetTransactionsBySQL(string sSQL);
		
		/// <summary>
		/// This abstract method that is to be implemented in the derived class will check and see if a transaction exists in the database.
		/// </summary>
		/// <param name="lTransID">Transaction ID</param>
		/// <returns>True or False</returns>
		public abstract bool DoesTransactionExist(long lTransID);

		/// <summary>
		/// This method will write a transaction to the database by ensuring the DATA, DATA_IDX, and DATA_LOG tables
		/// are all populated accordingly.  This method is the only method to be calling when saving individual transactions
		/// to the database.  The method uses retry logic where appropriate to ensure multiple storage attempts are made before
		/// failing the method.
		/// </summary>
		/// <param name="oTrans">Packet to save in the database.</param>
		/// <returns>True or False.</returns>
		public virtual bool StoreTransaction(Packet oTrans)
		{			
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(oTrans == null)
					return false;

				int iNumTries = 0;

				bool bResult = false;
				
				StringBuilder sb = null;

				while(iNumTries < DTPConstants.MAX_DB_ATTEMPTS)
				{
					//	Write the transaction to the database
					bResult = DBStoreTransactionRecord(oTrans);
					if(bResult == true)
					{
						if(iNumTries > 0)
						{
							sb = null;
							sb = new StringBuilder();
							if(sb != null)
							{
								sb.AppendFormat(DTPConfig.GetCommonText(109), iNumTries, oTrans.TransactionID);				
								Trace.WriteLineIf(Config.TraceInfo, sb.ToString());
							}
						}

						//	Populate the DATA_IDX table with the look up keys
						return StoreKeys(oTrans);						
					}

					sb = null;
					sb = new StringBuilder();
					if(sb != null)
					{
						sb.AppendFormat(DTPConfig.GetCommonText(110), iNumTries, oTrans.TransactionID);			
						Trace.WriteLineIf(Config.TraceWarning, sb.ToString());
					}
							
					iNumTries++;
				}	
				
				sb = null;
				sb = new StringBuilder();
				if(sb != null)
				{
					sb.AppendFormat(DTPConfig.GetCommonText(111), oTrans.TransactionID);		
					Trace.WriteLineIf(Config.TraceError, sb.ToString());
				}

				return false;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
			
			return false;
		}		


		/// <summary>
		/// This is an abstract method that must be implemented in the derived class.  This method is to be used
		/// to call ADO.NET provider logic to ensure the transaction is written to the database correctly and to 
		/// ensure the correct logic is implement to facialiate this action.  Do not call this method directly from
		/// products to save transactions to the database, use the StoreTransaction method instead.
		/// </summary>
		/// <param name="oTrans">Packet to save in the database.</param>
		/// <returns>True or False.</returns>
		public abstract bool DBStoreTransactionRecord(Packet oTrans);

		/// <summary>
		/// This is an additional helper method provided for saving Packet to the database and it might be migrated to the 
		/// derived class if Oracle can not use the same mechanism for passing package names.
		/// </summary>
		/// <param name="oTrans">Packet to save in the database.</param>
		/// <param name="sStoredProcedureName">Package or stored procedure to call.</param>
		/// <returns>True or False.</returns>
		public abstract bool DBStoreTransactionRecord(Packet oTrans, string sStoredProcedureName);

		/// <summary>
		/// This method will iterate through a collection of Packets and write them one at a time to the database.
		/// </summary>
		/// <param name="colTrans">Collection of Packets.</param>
		/// <returns>true or false</returns>
		public virtual bool StoreTransactions(PacketCollection colTrans)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(colTrans == null)
					return false;

				for(int i =0; i < colTrans.Count; i++)
				{
					bool bResult = false;
					bResult = StoreTransaction(colTrans.Item(i));
					if(bResult == false)
					{
						if(colTrans.Item(i) != null)
						{
							StringBuilder sb = null;
							sb =  new StringBuilder();
							if(sb != null)
							{
								sb.AppendFormat(DTPConfig.GetCommonText(112), colTrans.Item(i).TransactionID);
								Trace.WriteLineIf(Config.TraceError, sb.ToString());
							}
						}
					}
				}

				return true;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);				
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
			
			return false;
		}
		
		/// <summary>
		/// This abstract method that is to be implemented in the derived class will get the current database date and time.  
		/// If an error occurs, the date will return formatted as January 31, 1899 12:59:59 AM based on the LOCAL settings 
		/// of the server.
		/// </summary>
		/// <returns>DateTime</returns>
		public abstract DateTime GetCurrentDBDate();

		/// <summary>
		/// This method is not intended to be called outside the MSSQLPolyModelData.  It is used by the DTP internally!
		/// 
		///	This method will take an SQL command and execute it.  There is no BEGIN TRANS, COMMIT TRANS, 
		///	or ROLLBACK TRANS encasulating the entire operation.  This method is intended to be used to 
		///	execute INSERT, UPDATE, and DELETE SQL commands.  It is possible to execute other DDL and DML
		///	command; however, for those command that need to return a populated Packet use the ExecuteSQLReturnPacket() method.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.  Try to stick to INSERT, UPDATE, and DELETE SQL commands.</param>
		/// <param name="iProcessedRowCount">Number of rows processed. For UPDATE, INSERT, and DELETE statements, 
		/// this value is the number of rows affected by the command. For all other types of statements, the value is ModelFactory_ERROR.</param>
		/// <returns>true=success; false=an error occurred.</returns>
		public virtual bool ExecuteSQL(string sSQL, ref int iProcessedRowCount)
		{
			try
			{	
				Profiler.BeginInstrumentation();

				//	Check argument
				if(sSQL.Length == 0)
					return false;
				
				PolyModelData oDB = null;
				oDB = ModelFactory.CreatePolyModelData();
				if(oDB == null)
					return false;

				//	Execute the query
				iProcessedRowCount = oDB.ExecuteNonQuery(sSQL);

				return true;
			}
			catch(SqlException e)
			{
				LogSQLException(e);
			}
			catch(OleDbException e)
			{
				LogOleDbException(e);
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return false;
		}
		

		/// <summary>
		/// This method is not intended to be called outside the   It is used by the DTP internally!
		/// 
		///	This method will take an StringCollection of SQL commands and execute them one at a time.
		///	There is no BEGIN TRANS, COMMIT TRANS, or ROLLBACK TRANS encasulating the entire StringCollection
		///	as each string is processed one at a time through ExecuteSQL().  
		/// </summary>
		/// <param name="sSQLs">StringCollection of SQL command to execute</param>
		/// <returns>0 if successful, ModelFactory_ERROR for an unknown error, or a postive value identifying the string that encountered an error.</returns>
		public virtual int ExecuteSQLCollection(StringCollection scolSQL)
		{
			int iProcessedRowCount = 0;
			int iResult = 0;

			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(scolSQL == null)
					return DTPConstants.DB_ERROR;

				//	Process each SQL string one at a time
				for(int i = 0; i < scolSQL.Count; i++)
				{
					iProcessedRowCount = 0;

					//	Execute the query
					if(ExecuteSQL(scolSQL[i], ref iProcessedRowCount) == false)
						return i;	//	Got an error
				}	

				iResult = scolSQL.Count;
			
				return iResult;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return DTPConstants.DB_ERROR;
		}

		/// <summary>
		/// This method will automatically create and/or update sCounterName by incrementing the
		/// current value by 1.  This method calls another overloaded version of itself that sets
		/// the starting value for the counter to 1.
		/// 
		/// Do not call this method if you need a Transaction Identifier aka TRANS_ID - use GenerateTransactionID() instead.
		/// 
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <returns>Next counter value or ModelFactory_ERROR if an error occurs</returns>
		public virtual long GetNextCounterValue(string sCounterName)
		{		
			try
			{
				Profiler.BeginInstrumentation();

				return GetNextCounterValue(sCounterName, DTPConstants.DB_ERROR);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}
		

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will return the current value of a counter.
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <returns>Current counter value or ModelFactory_ERROR if an error occurs</returns>
		public abstract long GetCounterValue(string sCounterName);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will insert or update a counter with the 
		/// value passed in lValue.  If the counter does not exists, it is created.  
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <param name="lValue">New value of the counter</param>
		/// <returns>true if successful; false if an error occurred</returns>
		public abstract bool StoreCounterValue(string sCounterName, long lValue);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class  will automatically create and/or update 
		/// sCounterName by incrementing the current value by 1.  If the counter does not exists, it will be created and 
		/// the default value passed in will be used as the initialization value for the counter.  
		/// 
		/// Do not call this method if you need a Transaction Identifier aka TRANS_ID - use GenerateTransactionID() instead.
		/// 
		/// </summary>
		/// <param name="sCounterName">Name of the counter.</param>
		/// <param name="lDefaultValue">Default value of the counter.</param>
		/// <returns>Next counter value or ModelFactory_ERROR if an error occurs</returns>
		public abstract long GetNextCounterValue(string sCounterName, long lDefaultValue);

		/// <summary>
		/// This method will return a unique transaction identifier that typcially will map to the
		/// TRANS_ID DataColumn for the various ProductSchemas.
		/// </summary>
		/// <returns>Transaction ID or ModelFactory_ERROR if an error occurs</returns>
		public virtual long GenerateTransactionID()
		{
			try
			{
				Profiler.BeginInstrumentation();
				
				return GetNextCounterValue(DTPConstants.DEFAULT_TRANS_ID_NAME);	
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}		


		/// <summary>
		/// This abstract method that is to be implemented in the derived class will check and see if a specific counter exists in the database.
		/// </summary>
		/// <param name="sCounterName">Counter Name</param>
		/// <returns>True or False</returns>
		public abstract bool DoesCounterExist(string sCounterName);

		#region Key Builder Methods

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will remove all lookup keys from the database
		/// that are associated to the passed in Packet.  
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <returns>True or False</returns>
		public abstract bool RemoveAllKeys(Packet oTrans);
		
		/// <summary>
		/// This abstract method that is to be implemented in the derived class will insert all the lookup keys into the database
		/// that are associated to the passed in Packet.  
		/// </summary>
		/// <param name="oTrans">Packet</param>
		/// <returns>True or False</returns>
		public abstract bool StoreKeys(Packet oTrans);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class with insert a single lookup key into the database
		/// that is accociated to the passed in Packet.
		/// </summary>
		/// <param name="lTransID">Transaction ID</param>
		/// <param name="sKeyName">Look Up Key Name</param>
		/// <param name="sValue">Value of Look Up Key</param>
		/// <returns>True or False</returns>
		public abstract bool StoreKey(long lTransID, string sKeyName, string sValue);
		
		#endregion

		#region Helper Methods
		
		/// <summary>
		/// This method takes the raw XML form of a DataSet and populated a DataSet from it that is encasulated with a
		/// Packet.  The PacketTranslator is used to do the translation from XML into either a strongle typed DataSet
		/// or the generic DataSet object.
		/// </summary>
		/// <param name="sXML">XSD+DiffGram representation of a DataSet</param>
		/// <returns>PMPakcet hydrated from XML string</returns>
		public Packet XML2Packet(string sXML)
		{
			try
			{
				Profiler.BeginInstrumentation();

				PacketTranslator oTranslator = null;
				oTranslator =  new PacketTranslator();
				if(oTranslator == null)
					return null;

				return (Packet) oTranslator.Translate(sXML);
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}

			return null;
		}


		#endregion

		#region ADO.NET Exception Logging Methods

		/// <summary>
		/// This method will extract the contents of the SqlException and write the information to the appropriate
		/// trace location.
		/// </summary>
		/// <param name="e">SqlException</param>
		public virtual void LogSQLException(SqlException e)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(e == null)
					return;

				//	Create a string builder to contain the hydrated error message
				StringBuilder sbError = null;
				sbError = new System.Text.StringBuilder();
				
				//	Get a reference to the error collection the exception has
				SqlErrorCollection oErrors = null;
				oErrors = e.Errors;
				if(oErrors == null)
					return;

				//	Write infomation about where this occurred
				sbError.AppendFormat("Class: {0}", e.Class);	
				sbError.AppendFormat("\nError #{0}: {1} on line {2}.", e.Number, e.Message, e.LineNumber);	
				sbError.AppendFormat("\nError reported by {0} while connected to {1}", e.Source, e.Server);	
				sbError.AppendFormat("\nNeither record was written to database.");	
				sbError.AppendFormat("\nError occurred in method {0}.", e.TargetSite);	
				sbError.AppendFormat("\nErrors collection contains:\n\n");	

				//	Process each error one at a time
				for (int i=0; i < oErrors.Count; i++)
				{
					sbError.AppendFormat("Class: {0}\n", oErrors[i].Class);	
					sbError.AppendFormat("Error #{0}: {1} on line {2}.\n", oErrors[i].Number, oErrors[i].Message, oErrors[i].LineNumber);	
					sbError.AppendFormat("Error reported by {0} while connected to {1}.\n", oErrors[i].Source, oErrors[i].Server);	
				}
				
				sbError.AppendFormat("\nStack Trace: {0}", e.StackTrace);

				//	Dump error message to the appropriate location
				Trace.WriteLineIf(Config.TraceError, sbError.ToString());
			}
			catch(Exception e2)
			{
				Trace.WriteLineIf(Config.TraceError, e2.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}
		}		


		/// <summary>
		/// This method will extract the contents of the OleDbException and write the information to the appropriate
		/// trace location.
		/// </summary>
		/// <param name="e">SqlException</param>
		public virtual void LogOleDbException(OleDbException e)
		{
			try
			{
				Profiler.BeginInstrumentation();

				//	Check argument
				if(e == null)
					return;

				//	Create a string builder to contain the hydrated error message
				StringBuilder sbError = null;
				sbError = new System.Text.StringBuilder();
				
				sbError.AppendFormat("Error Code (HRESULT) {0} on line {1}.\n", e.ErrorCode, e.Message);	
				sbError.AppendFormat("\nError reported by {0} on Method {2}.", e.Source, e.TargetSite);		
				sbError.AppendFormat("\nNeither record was written to database.");	
				sbError.AppendFormat("\nErrors collection contains:");


				for (int i=0; i < e.Errors.Count; i++)
				{
					sbError.AppendFormat("Index #{0}\n", i);	
					sbError.AppendFormat("Message: {0}", e.Errors[i].Message);	
					sbError.AppendFormat("Native: {0}", e.Errors[i].NativeError.ToString());	
					sbError.AppendFormat("Source: {0}", e.Errors[i].Source);	
					sbError.AppendFormat("SQL: {0}\n", e.Errors[i].SQLState);	
				}

				sbError.AppendFormat("\nStack Trace: {0}", e.StackTrace);

				//	Dump error message to the appropriate location
				Trace.WriteLineIf(Config.TraceError, sbError.ToString());
			}
			catch(Exception e2)
			{
				Trace.WriteLineIf(Config.TraceError, e2.Message);
			}
			finally
			{
				Profiler.EndInstrumentation(new StringBuilder(this.ToString()).Append(".").Append(System.Reflection.MethodInfo.GetCurrentMethod().Name).ToString(), true);
			}			
		}


		#endregion
		
	}
}
