using System;
using System.Configuration;
using System.Diagnostics;
using System.Data;
using System.Text;
using DotNetPatterns.Common;

namespace DataTierPatterns.Data
{
	public abstract class PolyModelData : IDisposable
	{	
		
		/// <summary>
		/// Default constructor
		/// </summary>
		public PolyModelData()
		{
			if(Data == null)
				Data = new DataSet();	
		
			m_oiConnection = CreateConnection(ConnectionString);
		}
		
		/// <summary>
		/// Member variables that hold the value for connection parameters
		/// </summary>
		private string m_sServer = DTPConstants.DEFAULT_DBSERVER;			
		private string m_sCatalog = DTPConstants.DEFAULT_DBCATALOG;			
		private string m_sUserName = DTPConstants.DEFAULT_DBUSERNAME;
		private string m_sPassword = DTPConstants.DEFAULT_DBPASSWORD;			
		private string m_sApplicationName = DTPConstants.DEFAULT_DBAPPLICATIONAME;								
		private bool m_bNTAuth = DTPConstants.DEFAULT_DBNTAUTH;								
		private int m_iTimeOut = DTPConstants.DEFAULT_DBTIMEOUT;
		
		/// <summary>
		/// Member variable that holds the connection string
		/// </summary>
		private string m_sConnectionString = string.Empty;

		/// <summary>
		/// Member variables that holds various references to objects
		/// </summary>CommonData				
		private IDbConnection m_oiConnection = null;
		private IDbDataAdapter m_oiDataAdapter = null;			
		private IDbCommand m_oiCommand = null;			
		private DataSet m_oData = null;

		/// <summary>
		/// Overloaded constructor that will take a SQL command and setup a ADO.Net command object.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.</param>
		public PolyModelData(string sSQL) : this()
		{
			Data.DataSetName = "DotnetPatternsSchemaField";

			m_oiCommand = CreateCommand(sSQL);
		}

		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and setup a ADO.Net command object.
		/// </summary>
		/// <param name="sbSQL">SQL command to execute.</param>
		public PolyModelData(StringBuilder sbSQL) : this(sbSQL.ToString())
		{

		}


		/// <summary>
		/// Overloaded constructor that will take a SQL command and CommandType and setup a ADO.Net command object.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.</param>
		/// <param name="eType">IDbCommand CommandType</param>
		public PolyModelData(string sSQL, CommandType eType) : this(sSQL)
		{
			Command.CommandType = eType;
		}

		
		/// <summary>
		/// Overloaded constructor that will take a SQL command and CommandType and setup a ADO.Net command object.
		/// </summary>
		/// <param name="sbSQL">SQL command to execute.</param>
		/// <param name="eType">Command Type</param>
		public PolyModelData(StringBuilder sbSQL, CommandType eType) : this(sbSQL.ToString(), eType)
		{

		}

		
		#region Connection Settings Properties
			
		/// <summary>
		/// Property that maps to the name of the RDBMS server
		/// </summary>		
		public string Server
		{
			get	{	return m_sServer; }
			set { 	m_sServer = value; }
		}


		/// <summary>
		/// Property that maps to the database catalog to use.
		/// </summary>
		public string Catalog
		{
			get	{	return m_sCatalog;	}
			set	{	m_sCatalog = value; }
		}


		/// <summary>
		/// Property that maps to the username to use for database sessions (leave empty if you are using NT Authentication)
		/// The property will be ignored if NT Authentication (SSPI) is enabled.
		/// </summary>
		public string UserName
		{
			get	{	return m_sUserName;	}
			set { 	m_sUserName = value; }
		}


		/// <summary>
		/// Property that maps to the password to use for database sessions (leave empty if you are using NT Authentication)
		/// /// The property will be ignored if NT Authentication (SSPI) is enabled.
		/// </summary>
		public string Password
		{
			get {	return m_sPassword;	}
			set { 	m_sPassword = value; }
		}


		/// <summary>
		/// Property the specified the application name that the database will show for each database session.
		/// </summary>
		public string ApplicationName
		{
			get	{	return m_sApplicationName;	}
			set { 	m_sApplicationName = value; }
		}


		/// <summary>
		/// Property that specified if NT Authentication is enable.  
		/// True = Use NT Authentication
		/// False = Do not use NT Authentication
		/// </summary>
		virtual public bool NTAuth
		{
			get	{	return m_bNTAuth;	}
			set { 	m_bNTAuth = value;	}
		}


		/// <summary>
		/// Property that specifies how long before the attempt to connect to the database times out.
		/// </summary>
		public int TimeOut
		{
			get	{	return m_iTimeOut;	}
			set {	m_iTimeOut = value; }
		}		


		#endregion

		/// <summary>
		/// Read only property that contains the connection string.
		/// </summary>
		public string ConnectionString
		{
			get
			{
				if(m_sConnectionString == string.Empty)
					m_sConnectionString = GenerateConnectionString();
				
				return m_sConnectionString;
			}
		}


		/// <summary>
		/// Read only property that returns true if the connection to the database is open or
		/// false if it is not.
		/// </summary>
		virtual public bool Connected
		{
			get
			{
				if(Connection == null)
					return false;

				if(Connection.State == ConnectionState.Open)
					return true;

				return false;
			}
		}


		/// <summary>
		/// Read only property that returns the current connection state of the database connection
		/// </summary>
		virtual public ConnectionState ConnectionState
		{
			get
			{
				return Connection.State;
			}
		}
		

		/// <summary>
		/// Read only property that points to the DataConnection.
		/// 
		/// </summary>
		public IDbConnection Connection
		{
			get { return m_oiConnection; }
		}


		/// <summary>
		/// Read only property that points to the DataAdapter
		/// </summary>
		public IDbDataAdapter DataAdapter
		{
			get { return m_oiDataAdapter; }
		}


		/// <summary>
		/// Property used to set/get the encasulated DataSet
		/// </summary>
		public DataSet Data
		{
			get { return m_oData; }
			set { m_oData = value; }
		}


		/// <summary>
		/// Property that maps to the encasulated DataSet's Name Property
		/// </summary>
		public string DataSetName
		{
			get { return Data.DataSetName; }
			set { Data.DataSetName = value;	}
		}


		/// <summary>
		/// Read Only property for the current ADO.NET Command object
		/// </summary>
		public IDbCommand Command
		{
			get { return m_oiCommand; }
		}


		/// <summary>
		/// Property for the current ADO.NET command Text property
		/// </summary>
		virtual public string CommandText
		{
			get { return Command.CommandText; }
			set 
			{ 
				Command.CommandText = value; 
			}
		}


		/// <summary>
		/// Property for the ADO.NET Command CommandType property
		/// </summary>
		public CommandType CommandType
		{
			get { return Command.CommandType; }
			set	{ Command.CommandType = value; }
		}
	
		/// <summary>
		/// Reads the database settings from web.config appsetting section to use to connect to the RDBMS.
		/// </summary>
		/// <returns>true = success; false = failed</returns>		
		virtual public bool ReadConfigurationSettings()
		{
			bool bResult = false;

			try
			{
				//Server = Config.GetAppSetting(DTPConstants.DB_TYPE_TEXT);
				ApplicationName = Config.GetAppSetting(DTPConstants.DB_APPNAME_TEXT);
				Server = Config.GetAppSetting(DTPConstants.DB_SERVER_TEXT);
				Catalog = Config.GetAppSetting(DTPConstants.DB_CATALOG_TEXT);
				UserName = Config.GetAppSetting(DTPConstants.DB_USERNAME_TEXT);
				Password = Config.GetAppSetting(DTPConstants.DB_PASSWORD_TEXT);
				NTAuth = Convert.ToBoolean(Config.GetAppSetting(DTPConstants.DB_NTAUTH_TEXT));
				TimeOut = Convert.ToInt32(Config.GetAppSetting(DTPConstants.DB_TIMEOUT_TEXT));

				bResult = true;
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}

			return bResult;	
		}		

		/// <summary>
		/// TODO: Saves the database settings to use to connect to the RDBMS in the web.config.
		/// </summary>
		/// <returns>true = success; false = failed</returns>
		virtual public bool SaveConfigurationSettings()
		{
			return false;
		}

		/// <summary>
		/// This method will generate a connection string to use to connect to the RDBMS
		/// </summary>
		/// <returns>A connection string.  If an error occurs the string will be string.empty</returns>
		virtual public string GenerateConnectionString()
		{
			ReadConfigurationSettings();

			//	Create a string builder to use for assembling the connection string
			StringBuilder sbConnectionString = null;
			sbConnectionString = new System.Text.StringBuilder();
			if(sbConnectionString == null)
				return string.Empty;
			
			//	Append each element of the connection string 
			sbConnectionString.AppendFormat("Application Name={0};", ApplicationName);
			sbConnectionString.AppendFormat("Connect Timeout={0};", TimeOut.ToString());
			sbConnectionString.AppendFormat("Server={0};", Server);
			sbConnectionString.AppendFormat("Database={0};", Catalog);
			sbConnectionString.Append("Persist Security Info=False;");

			//	See it we need to enable NT Authentication (SSPI)
			if(NTAuth == true)
				sbConnectionString.Append("Integrated Security=sspi;");	//	Yes
			else
			{
				//	No, use SQL Server Login Method (UserName and Password)
				sbConnectionString.AppendFormat("Password={0};", Password);
				sbConnectionString.AppendFormat("User ID={0};", UserName);
			}
				
			//	Return the fully assembled connection string
			return sbConnectionString.ToString();
		}


		/// <summary>
		/// This method will transform the value for a specific DataRow's DataColumn from its
		/// string format into a DataSet.  It is assumed you are only working on the encasulated 
		/// DataSet.  If you need to transform a different DataSet, create another instance of this object,
		/// assign the Data property to the DataSet you want to work on and call this method.  This method assumes
		/// you want to work with DataRow 0.
		/// </summary>
		/// <param name="sTable">DataTable name.</param>
		/// <param name="sColumn">DataColumn name.</param>
		/// <returns>DataSet hydrated from the DataRow's Column Value or null if a error occurred.</returns>
		virtual public DataSet RowColumnValue2DataSet(string sTable, string sColumn)
		{
			return RowColumnValue2DataSet(sTable, sColumn, 0);
		}

		
		/// <summary>
		/// This method will transform the value for a specific DataRow's DataColumn from its
		/// string format into a DataSet.  It is assumed you are only working on the encasulated 
		/// DataSet.  If you need to transform a different DataSet, create another instance of this object,
		/// assign the Data property to the DataSet you want to work on and call this method.
		/// </summary>
		/// <param name="sTable">DataTable name.</param>
		/// <param name="sColumn">DataColumn name.</param>
		/// <param name="iRow">DataRow to access.</param>
		/// <returns>DataSet hydrated from the DataRow's Column Value or null if a error occurred.</returns>
		/// 
		virtual public DataSet RowColumnValue2DataSet(string sTable, string sColumn, int iRow)
		{
			string sXML = string.Empty;
			sXML = (string) this[sTable, sColumn, iRow];

			DataSet oDS = null;
			oDS = new DataSet();
			if(oDS == null)
				return null;

			ReadXml(oDS, sXML);

			return oDS;
		}


		/// <summary>
		/// Opens the connection to the database
		/// </summary>
		virtual public void Open()
		{
			if(Connection != null)
			{
				if(Connected == true)
					Close();

				Connection.Open();
			}			
		}


		/// <summary>
		/// Closes the connection to the database
		virtual public void Close()
		{
			if(Connection != null)
				Connection.Close();
		}


		/// <summary>
		/// Simply delegates to the close method to cleanup
		/// </summary>
		virtual public void Dispose()
		{
			if(Connection != null)
				Close();
		}


		/// <summary>
		/// Hydrate the encasulated Dataset with the XML passed to the method
		/// </summary>
		/// <param name="sSchema">String containing the XSD XML Schema and instance data.</param>
		virtual public void ReadXml(string sXML)
		{
			if(Data == null)
				Data = new DataSet();
			
			if(Data != null)
				ReadXml(Data, sXML);
		}

		
		/// <summary>
		/// Hydrate the passed in DataSet from the passed in XML
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sXML">String containing the XSD XML Schema and instance data.</param>
		virtual public void ReadXml(DataSet oDS, string sXML)
		{
			if(oDS == null)
				return;

			System.IO.StringReader oReader = null;
			oReader = new System.IO.StringReader(sXML);
			if(oReader != null)
				oDS.ReadXml(oReader);				
		}

		
		/// <summary>
		/// Hydrate the encasulated DataSet with the passed in XML using the specified ReadMode
		/// </summary>
		/// <param name="sXML">String containing the XSD XML Schema and instance data.</param>
		/// <param name="eMode">XMLRead mode that describes how the XSD XML Schema and instance data is to be read by the XmlReader.</param>
		virtual public void ReadXml(string sXML, XmlReadMode eMode)
		{
			if(Data == null)
				Data =  new DataSet();

			ReadXml(Data, sXML, eMode);
		}

		
		/// <summary>
		/// Hydrate the passed in DataSet from the passed in XML using the XMLRead mode specified
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sXML">String containing the XSD XML Schema and instance data.</param>
		/// <param name="eMode">XMLRead mode that describes how the XSD XML Schema and instance data is to be read by the XmlReader</param>
		virtual public void ReadXml(DataSet oDS, string sXML, XmlReadMode eMode)
		{
			if(oDS == null)
				return;

			System.IO.StringReader oReader = null;
			oReader = new System.IO.StringReader(sXML);
			if(oReader != null)
				oDS.ReadXml(oReader, eMode);
		}

		
		/// <summary>
		/// Uses passed in XML schema to hydrate the current encasulated DataSet
		/// </summary>
		/// <param name="sSchema">String containing XSD XML Schema.</param>
		virtual public void ReadXmlSchema(string sSchema)
		{
			Data = null;
			Data = new DataSet();

			ReadXmlSchema(Data, sSchema);
		}

		
		/// <summary>
		/// Uses passed in XML schema to hydrate the passed in DataSet
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sSchema">String containing XSD XML Schema.</param>
		virtual public void ReadXmlSchema(DataSet oDS, string sSchema)
		{
			if(oDS == null)
				return;
			System.IO.StringReader oReader = null;
			oReader = new System.IO.StringReader(sSchema);
			if(oReader != null)
				oDS.ReadXmlSchema(oReader);
		}

		
		/// <summary>
		/// Returns the XSD Schema of the encasulated DataSet
		/// </summary>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXmlSchema()
		{
			if(Data == null)
				Data =  new DataSet();

			return WriteXmlSchema(Data);
		}

		
		/// <summary>
		/// Returns the XSD Schema of the passed in DataSet
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXmlSchema(DataSet oDS)
		{
			if(oDS == null)
				return string.Empty;

			System.IO.StringWriter oSW = null;
			oSW = new System.IO.StringWriter();
			if(oSW == null)
				return string.Empty;

			oDS.WriteXmlSchema(oSW);

			return oSW.ToString();
		}

		
		/// <summary>
		/// Returns the XSD Schema and instance data of the encasulated DataSet
		/// </summary>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXml()
		{
			if(Data == null)
				Data = new DataSet();
	
			if(Data != null)
				return WriteXml(Data);

			return string.Empty;
		}

		
		/// <summary>
		/// Returns the XSD Schema and instance data of the passed in DataSet
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXml(DataSet oDS)
		{
			if(oDS == null)
				return string.Empty;

			System.IO.StringWriter oSW = null;
			oSW = new System.IO.StringWriter();
			if(oSW == null)
				return string.Empty;

			oDS.WriteXml(oSW);

			return oSW.ToString();
		}

		
		/// <summary>
		/// Returns the XSD Schema and instance data of the encasulated DataSet using the specified XmlWriteMode
		/// </summary>
		/// <param name="eMode">XmlWriteMode mode that describes how the XSD XML Schema and instance data is to be written by the XMLWriter.</param>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXml(XmlWriteMode eMode)
		{
			if(Data == null)
				Data = new DataSet();

			return WriteXml(Data, eMode);
		}

		
		/// <summary>
		/// Returns the XSD Schema and instance data of the passed in DataSet using the specified XmlWriteMode
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="eMode">XmlWriteMode mode that describes how the XSD XML Schema and instance data is to be written by the XMLWriter.</param>
		/// <returns>String containing the XSD XML Schema or if an error occurs string.empty.</returns>
		virtual public string WriteXml(DataSet oDS, XmlWriteMode eMode)
		{
			if(oDS == null)
				return string.Empty;

			System.IO.StringWriter oSW = null;
			oSW = new System.IO.StringWriter();
			if(oSW == null)
				return string.Empty;

			oDS.WriteXml(oSW, eMode);

			return oSW.ToString();
		}

		
		/// <summary>
		/// Adds a blank row to the encasulated DataSet.
		/// </summary>
		/// <param name="sTable">Name of the DataTable to add a blank row to.</param>
		/// <returns>True or False</returns>
		virtual public DataRow AddBlankRow(string sTable)
		{
			if(Contains(sTable) == false)
				return null;

			DataRow oRow = null;
			oRow = this[sTable].NewRow();
			if(oRow == null)
				return null;

			this[sTable].Rows.Add(oRow);
			
			return oRow;
		}	
	
		/// <summary>
		/// Adds a blank row to the passed in DataTable
		/// </summary>
		/// <param name="oTable">DataTable object to add a new DataRow too.</param>
		/// <returns>True or False</returns>
		virtual public bool AddBlankRow(DataTable oTable)
		{
			if(oTable == null)
				return false;

			DataRow oRow = null;
			oRow = oTable.NewRow();
			if(oRow == null)
				return false;

			oTable.Rows.Add(oRow);			

			return true;
		}	

	
		/// <summary>
		/// Determines if the passed in DataTable name exists in the encasulated DataSet
		/// </summary>
		/// <param name="sTableName">DataTable name.</param>
		/// <returns>True or False</returns>
		virtual public bool Contains(string sTableName)
		{
			return Contains(Data, sTableName);
		}

		/// <summary>
		/// Determines if the passed in DataTable name exists in the passed in DataSet
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sTableName">DataTable name.</param>
		/// <returns>True or False</returns>
		virtual public bool Contains(DataSet oDS, string sTableName)
		{
			if(oDS == null)
				return false;

			if(oDS.Tables.Contains(sTableName) == false)
				return false;

			return true;
		}
		

		/// <summary>
		/// This method will create an IDbCommand object and assign a SQL Command to it.
		/// </summary>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <returns>True or False</returns>
		virtual public bool PrepareCommand(string sSQL)
		{
			if(Connection == null)
			{
				m_oiConnection = CreateConnection(ConnectionString);
				if(Connection == null)
					throw new ModelFactoryException();
			}

			if(Command == null)
			{
				m_oiCommand = CreateCommand(sSQL);
				if(Command == null)
					return false;
			}

			Command.CommandText = sSQL;

			return true;
		}
		

		/// <summary>
		/// This method is intended to adds or refreshes rows for a DataTable in the encasulated DataSet.  
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		/// </summary>
		/// <returns>
		/// The number of rows successfully refreshed to the encasulated DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int Fill()
		{
			return Fill(Data);
		}

		
		/// <summary>
		/// This method is intended to adds or refreshes rows for a DataTable in the encasulated DataSet.  
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <returns>
		/// The number of rows successfully refreshed to the passed in DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int Fill(DataSet oDS)
		{
			if(oDS == null)
				return DTPConstants.DB_ERROR;

			return DataAdapter.Fill(oDS);
		}

		
		/// <summary>
		/// This method is intended to adds or refreshes rows for a DataTable in the encasulated DataSet.
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		/// </summary>
		/// <param name="sTableName">Name of DataTable to fill.</param>
		/// <returns>
		/// The number of rows successfully refreshed to the encasulated DataTable. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int Fill(string sTableName)
		{
			return Fill(Data, sTableName);
		}
	

		/// <summary>
		///	This method is intended to adds or refreshes rows for the DataTable in the encasulated DataSet based on the 
		///	results of the passed in query.
		/// </summary>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <returns>
		/// The number of rows successfully refreshed in the encasulated in DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int ExecuteFillQuery(string sSQL)
		{
			return ExecuteFillQuery(Data, sSQL);
		}


		/// <summary>
		///	This method is intended to adds or refreshes rows for the DataTable in the passed in DataSet based on the 
		///	results of the passed in query.
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sSQL">SQL Command to execute.</param>		
		/// <returns>
		/// The number of rows successfully refreshed in the passed in DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int ExecuteFillQuery(DataSet oDS, string sSQL)
		{
			if(oDS == null)
				return DTPConstants.DB_ERROR;

			if(Connection == null)
			{
				m_oiConnection = CreateConnection(ConnectionString);
				if(Connection == null)
					throw new ModelFactoryException();
			}
			
			if(Connected == false)
				Open();

			if(Connected == false)
				throw new ModelFactoryException();

			if(Command == null)
			{
				m_oiCommand = CreateCommand(sSQL);
				if(Command == null)
					return DTPConstants.DB_ERROR;	
			}
			else
				m_oiCommand.CommandText = sSQL;

			if(DataAdapter == null)
			{
				m_oiDataAdapter = CreateDataAdapter();
				if(DataAdapter == null)
					return DTPConstants.DB_ERROR;
			}

			DataAdapter.SelectCommand = Command;

			return Fill(oDS);
		}


		/// <summary>
		///	This method is intended to adds or refreshes rows for the DataTable in the encasulated DataSet based on the 
		///	results of the passed in query.
		/// </summary>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <param name="sTableName"></param>
		/// <returns>
		/// The number of rows successfully refreshed in the encasulated DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int ExecuteFillQuery(string sSQL, string sTableName)
		{
			return ExecuteFillQuery(Data, sSQL, sTableName);
		}


		/// <summary>
		///	This method is intended to adds or refreshes rows for the DataTable in the passed in DataSet based on the 
		///	results of the passed in query.
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <param name="sTableName">DataTable name to apply Fill() on.</param>
		/// <returns>
		/// The number of rows successfully refreshed in the passed in DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns>
		virtual public int ExecuteFillQuery(DataSet oDS, string sSQL, string sTableName)
		{
			if(oDS == null)
				return DTPConstants.DB_ERROR;

			if(Connection == null)
			{
				m_oiConnection = CreateConnection(ConnectionString);
				if(Connection == null)
					throw new ModelFactoryException();
			}
			
			if(Connected == false)
				Open();
			
			if(Connected == false)
				throw new ModelFactoryException();

			if(Command == null)
			{
				m_oiCommand = CreateCommand(sSQL);
				if(Command == null)
					return DTPConstants.DB_ERROR;
			}
			else
				m_oiCommand.CommandText = sSQL;

			if(DataAdapter == null)
			{
				m_oiDataAdapter = CreateDataAdapter();
				if(DataAdapter == null)
					return DTPConstants.DB_ERROR;
			}

			DataAdapter.SelectCommand = Command;
			
			return Fill(oDS, sTableName);
		}


		/// <summary>
		/// Executes the query, and returns the first column of the first row in the resultset returned by the query. 
		/// Extra columns or rows are ignored.
		/// 
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		///	
		/// </summary>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <returns>The first column of the first row in the resultset or null if an error occurred.</returns>
		virtual public object ExecuteScalar(string sSQL)
		{
			if(Connection == null)
			{
				m_oiConnection = CreateConnection(ConnectionString);
				if(Connection == null)
					throw new ModelFactoryException();
			}

			if(Connected == false)
				Connection.Open();

			if(Connected == false)
				throw new ModelFactoryException();

			if(Command == null)
			{
				m_oiCommand = CreateCommand(sSQL);
				if(Command == null)
					return null;
			}
			else
				m_oiCommand.CommandText = sSQL;


			return Command.ExecuteScalar();
		}


		/// <summary>
		/// Executes the query, and returns the first column of the first row in the resultset returned by the query. 
		/// Extra columns or rows are ignored.
		/// 
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		///	
		/// </summary>
		/// <param name="sbSQL">SQL Command to execute.</param>
		/// <returns>The first column of the first row in the resultset or null if an error occurred.</returns>
		virtual public object ExecuteScalar(StringBuilder sbSQL)
		{
			return ExecuteScalar(sbSQL.ToString());
		}


		/// <summary>
		/// Executes the query, and returns the first column of the first row in the resultset returned by the query. 
		/// Extra columns or rows are ignored.
		/// 
		///	This method may need to be overridden in the derived class due to the different overloads
		///	and behavior the .NET native and OLEDB providers provide.
		///	
		/// </summary>
		/// <returns>The first column of the first row in the resultset or null if an error occurred.</returns>
		virtual public object ExecuteScalar()
		{
			return Command.ExecuteScalar();
		}


		/// <summary>
		/// Executes a SQL statement against the Connection object of an ADO.NET data provider, and returns the number of rows affected.
		/// </summary>
		/// <param name="sSQL">SQL Command to execute.</param>
		/// <returns>The number of rows affected or ModelFactory_ERROR if an error occurred.</returns>
		virtual public int ExecuteNonQuery(string sSQL)
		{
			if(Connection == null)
			{
				m_oiConnection = CreateConnection(ConnectionString);
				if(Connection == null)
					throw new ModelFactoryException();
			}

			if(Connected == false)
				Connection.Open();

			if(Connected == false)
				throw new ModelFactoryException();

			if(Command == null)
			{
				m_oiCommand = CreateCommand(sSQL);
				if(Command == null)
					return DTPConstants.DB_ERROR;
			}
			else
				m_oiCommand.CommandText = sSQL;

			return Command.ExecuteNonQuery();
		}


		/// <summary>
		/// Executes a SQL statement against the Connection object of an ADO.NET data provider, and returns the number of rows affected.
		/// </summary>
		/// <param name="sbSQL">SQL Command to execute.</param>
		/// <returns>The number of rows affected or ModelFactory_ERROR if an error occurred.</returns>
		virtual public int ExecuteNonQuery(StringBuilder sbSQL)
		{
			return ExecuteNonQuery((string) sbSQL.ToString());
		}


		/// <summary>
		/// Executes a SQL statement against the Connection object of an ADO.NET data provider, and returns the number of rows affected.
		/// </summary>
		/// <returns>The number of rows affected or ModelFactory_ERROR if an error occurred.</returns>
		virtual public int ExecuteNonQuery()
		{
			if(Connection == null)
				throw new ModelFactoryException();

			if(Connected == false)
				Connection.Open();

			if(Connected == false)
				throw new ModelFactoryException();
			
			if(Command == null)
				return DTPConstants.DB_ERROR;

			return Command.ExecuteNonQuery();
		}


		#region Abstract ADO.NET Wrapper Methods

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// create the correct ADO.NET connection object for the specific ADO.NET Native or OLEDB provider
		/// used by the derived class.
		/// </summary>
		/// <returns>A IDbConnection interface reference to a ADO.NET Connection object or null if an error occurred.</returns>
		abstract public IDbConnection CreateConnection();

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// create the correct ADO.NET connection object for the specific ADO.NET Native or OLEDB provider
		/// used by the derived class.
		/// </summary>
		/// <param name="sConnectionString">Connection string to use during Connection object construction.</param>
		/// <returns>A IDbConnection interface reference to a ADO.NET Connection object or null if an error occurred.</returns>
		abstract public IDbConnection CreateConnection(string sConnectionString);

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// create the correct ADO.NET DataAdaptor object for the specific ADO.NET Native or OLEDB provider
		/// used by the derived class.
		/// </summary>
		/// <returns>A IDbDataAdapter interface reference to a ADO.NET DataAdapter object or null if an error occurred.</returns>
		abstract public IDbDataAdapter CreateDataAdapter();

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// create the correct ADO.NET Command object for the specific ADO.NET Native or OLEDB provider
		/// used by the derived class.
		/// </summary>
		/// <returns>A IDbCommand interface reference to a ADO.NET Command object or null if an error occurred.</returns>
		abstract public IDbCommand CreateCommand();

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// create the correct ADO.NET Command object for the specific ADO.NET Native or OLEDB provider
		/// used by the derived class.
		/// </summary>
		/// <param name="sSQL">SQL command to use during ADO.NET Command construction</param>
		/// <returns>A IDbCommand interface reference to a ADO.NET Command object or null if an error occurred.</returns>
		abstract public IDbCommand CreateCommand(string sSQL);

		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to 
		/// adds or refreshes rows for the DataTable in the passed in DataSet.  This method is abstract 
		/// since there are numerous overloads for this method for both the OleDB and native .NET providers.
		/// </summary>
		/// <param name="oDS">DataSet to receive action of method.</param>
		/// <param name="sTableName">Name of DataTable to fill.</param>
		/// <returns>
		/// The number of rows successfully refreshed to the encasulated DataSet. This does not include rows affected 
		/// by statements that do not return rows.  If ModelFactory_ERROR is returned an error occurred.
		/// </returns> 
		abstract public int Fill(DataSet oDS, string sTableName);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will adds an IDbDataParameter to the IDbCommand object
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		abstract public IDbDataParameter AddDataParameter(string sName, object oDataType);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will adds an IDbDataParameter to the IDbCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="iSize">Gets or sets the maximum size, in bytes, of the data within the column.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		abstract public IDbDataParameter AddDataParameter(string sName, object oDataType, int iSize);		

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will adds an IDbDataParameter to the IDbCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="eDirection">Value indicating whether the parameter is input-only, output-only, bidirectional, or a stored procedure return value parameter.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		abstract public IDbDataParameter AddDataParameter(string sName, object oDataType, ParameterDirection eDirection);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class will dd an IDbDataParameter to the IDbCommand object.
		/// </summary>
		/// <param name="sName">Parameter name</param>
		/// <param name="oDataType">Parameter database datatype</param>
		/// <param name="iSize">Gets or sets the maximum size, in bytes, of the data within the column.</param>
		/// <param name="eDirection">Value indicating whether the parameter is input-only, output-only, bidirectional, or a stored procedure return value parameter.</param>
		/// <returns>IDbDataParameter or null if an error occurred.</returns>
		abstract public IDbDataParameter AddDataParameter(string sName, object oDataType, int iSize, ParameterDirection eDirection);

		/// <summary>
		/// Returns the a IDbDataParameter contained in the IDbCommand object.
		/// </summary>
		/// <param name="sName">Name of IDbDataParameter to return.</param>
		/// <returns>IDbDataParameter</returns>
		abstract public IDbDataParameter GetDataParameter(string sName);

		/// <summary>
		/// This abstract method that is to be implemented in the derived class returns a IDbDataParameter contained in the 
		/// IDbCommand object.
		/// </summary>
		/// <param name="sName">Name of IDbDataParameter.</param>
		/// <returns>IDbDataParameter</returns>
		abstract public object GetDataParameterValue(string sName);

		#endregion

		#region Indexers

		/// <summary>
		/// This property will return the value of a specific DataColumn within a specific DataRow within a DataTable
		/// when reading the property.  This property will also set the specific DataColumn value within a specific DataRow
		/// within a DataTable.  If the specific DataRow does not exist in the DataSet during Set property calls, a exception will
		/// be thrown.  **NOTE: Unlike the previous implementation of ModelFactory, this method does not insert blank rows automatically when it 
		/// do not exists.  Call AddBlankRow() to add a row before calling this method for new rows, or if constraints are involved 
		/// use the normal DataTable methods for adding new rows.
		/// </summary>
		/// <param name="sTable">Name of DataTable</param>
		/// <param name="sColumn">Name of DataColumn within DataTable</param>
		/// <param name="nRow">Index of the DataRow to access within the DataTables DataRowCollection</param>
		/// <returns>Value of the specific DataColumn within DataRow or throw an exception if an error occurs.</returns>
		virtual public object this[string sTable, string sColumn, int nRow]
		{
			get 
			{
				DataRow row = null;
				object oReturn = null;

				// first check to see if row exists else return null, otherwise we'll get a NullPtr exception
				if (Data.Tables[sTable].Rows.Count > 0)	
				{
					// grab the first row found and set the value
					row = Data.Tables[sTable].Rows[nRow];
					oReturn = row[sColumn];
				}
					
				return oReturn;
			}
			set 
			{
				DataRow row = null;				
				row = Data.Tables[sTable].Rows[nRow];
				row[sColumn] = value;
			}
		}


		/// <summary>
		/// This property will return the value of a specific DataColumn within DataRow 0 within a DataTable
		/// when reading the property.  This property will also set the specific DataColumn value within DataRow 0
		/// within a DataTable.  **NOTE: Unlike the previous implementation of ModelFactory, this method does not insert blank rows automatically when they 
		/// do not exists.
		/// </summary>
		/// <param name="sTable">Name of DataTable</param>
		/// <param name="sColumn">Name of DataColumn within DataTable</param>
		/// <returns>Value of the specific DataColumn within DataRow or throw an exception if an error occurs.</returns>
		virtual public object this[string sTable, string sColumn]
		{
			get {	return this[sTable, sColumn, 0];	}	
			set {	this[sTable, sColumn, 0] = value;	}	
		}


		/// <summary>
		/// This read only property will return a specific DataTable within the encasulated DataSet.
		/// </summary>
		/// <param name="sTable">Name of DataTable</param>
		/// <returns>Value of the specific DataTable within the encasulated DataSet or throw an exception if an error occurs.</returns>
		virtual public DataTable this[string sTable]
		{
			get {	return Data.Tables[sTable];	}
		}


		#endregion


	}
}
