using System;
using System.Threading;

namespace Delays
{
	/// <summary>
	/// Should be used as a semi-boolean return value 
	/// for asynchronous transactions.  The addition 
	/// of the two additional values allow us to show 
	/// that the transaction is not complete yet or 
	/// that we just don't know what the status of the
	/// thread's execution is.
	/// </summary>
	public enum AsyncReturn : int
	{
		/// <summary>
		/// We have no idea how we got here.
		/// </summary>
		Unknown = -2,
		/// <summary>
		/// The transaction is not complete, yet.
		/// </summary>
		Running = -1,
		/// <summary>
		/// The transaction has completed unsuccessfully
		/// </summary>
		CompletedUnsuccessfully = 0,
		/// <summary>
		/// The transaction has completed successfully
		/// </summary>
		CompletedSuccessfully = 1
	}

	

	/// <summary>
	/// Will manager the process of creating a separate thread 
	/// that can be periodically polled to check for completion
	/// of the operation.
	/// </summary>
	public class PollableThreadManager
	{
		/// <summary>
		/// Begin the async operation.  Wait 1 second for the 
		/// operation to complete before returning.
		/// </summary>
		/// <returns>The status of the operation after 1 second.</returns>
		public AsyncReturn BeginExecution()
		{
			return BeginExecution(1000);
		}

		/// <summary>
		/// Begin the async operation.  Wait the specified number
		/// of milliseconds for the operation to complete before
		/// returning.
		/// </summary>
		/// <param name="MillisecondsToWait">0 indicates no wait.  -1 indicates infinite wait.</param>
		/// <returns>The status of the operation after the specified number of milliseconds.</returns>
		public AsyncReturn BeginExecution(int MillisecondsToWait)
		{
			// By default, we don't have an error
			mbError = false;

			// Start the thread
			ThreadStart startThread = new ThreadStart(DoExecution);
			mExecution = new Thread(startThread);
			mExecution.Start();			

			// Give it a chance to complete before we go on
			return WaitForCompletion(MillisecondsToWait);
		}

		/// <summary>
		/// Wait the specified number of milliseconds for the 
		/// operation to complete then return the current status
		/// of the operation.
		/// </summary>
		/// <param name="MillisecondsToWait">0 indicates no wait.  -1 indicates infinite wait.</param>
		/// <returns>The status of the operation after the specified number of milliseconds.</returns>
		public AsyncReturn WaitForCompletion(int MillisecondsToWait)
		{
			// If we don't have a thread, we have problem
			if (mExecution == null) 
				return AsyncReturn.Unknown;

			// If the thread is either stopped or not running
			// yet, wait for it to complete
			if (0 == (mExecution.ThreadState & 
						(ThreadState.Stopped | ThreadState.Unstarted)))
			{
				if (!mExecution.Join(MillisecondsToWait))
				{
					// We didn't finish yet, let the caller know
					return AsyncReturn.Running;
				}
			}

			// We have an error in execution
			if (mbError) 
				return AsyncReturn.CompletedUnsuccessfully;

			// Success!!!
			return AsyncReturn.CompletedSuccessfully;
		}
		
		
		/// <summary>
		/// How we know if we were successful or not.  Not exposed since the caller
		/// will get a "CompletedUnsuccessfully" return on a failure.
		/// </summary>
		private bool mbError = false;

		/// <summary>
		/// Our actual thread of execution
		/// </summary>
		private Thread mExecution;

		/// <summary>
		/// Exception thrown during execution (if any).
		/// </summary>
		private Exception mException = null;
		/// <summary>
		/// Exception is thrown during execution, if any.
		/// </summary>
		public Exception ExceptionThrown
		{
			get { return mException; }
		}

		/// <summary>
		/// The ID provided by the caller
		/// </summary>
		private int mID;
		/// <summary>
		/// The ID provided by the caller
		/// </summary>
		public int ID
		{
			get { return mID; }
		}
		/// <summary>
		/// The value to be factored.
		/// </summary>
		private ulong mToFactor;
		/// <summary>
		/// Holds the result of the operation
		/// </summary>
		private string mResult;
		/// <summary>
		/// The comma delimited factors.
		/// </summary>
		public string Result 
		{
			get { return mResult; }
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="ToFactor">The value to factor.</param>
		public PollableThreadManager(int ID, ulong ToFactor)
		{
			mID = ID;
			mToFactor = ToFactor;
			mbError = false;
		}

		/// <summary>
		/// Our long running operation that will execute on 
		/// a separate thread.
		/// </summary>
		protected void DoExecution()
		{
			try
			{
				// Get the answer
				mResult = Factorer.Factor(mToFactor);
			}
			catch(Exception ex)
			{
				// Store the exception
				mException = ex;
				// ...and the fact that we failed.
				mbError = true;
			}
		}

	}
}
