// Copyright (c) 2002-2003 Brian Eshelman, EshelTech Consulting.
// All Rights Reserved.  Developed for Dot Net Design Patterns.
// 
// Permission to use, copy, modify, and distribute this 
// software and its documentation for NON-COMMERCIAL purposes
// and without fee is hereby granted provided that this 
// copyright notice appears in all copies. 
//
// THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT 
// THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR 
// NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY 
// DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, 
// MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
// Please send any comments to beshelman@esheltech.com
//

using System;
using System.Reflection;
using System.Collections;

namespace TestBed
{
	/// <summary>
	/// Provide a pattern to simplify dynamic loading, interrogation, and caching of the types contained in an assembly.  
	/// </summary>
	public class CachingTypeLoader
	{
		private SortedList collTypes = null;
			
		public CachingTypeLoader()
		{
			collTypes = new SortedList();
		}

		/// <summary>
		/// Load all of the types for the specified Assembly
		/// </summary>
		/// <param name="AssemblyPath"></param>
		/// <returns></returns>
		public IList LoadTypeList(string AssemblyPath)
		{
			// Load the assembly into memory
			Assembly theAssembly = Assembly.LoadFrom(AssemblyPath);
			// Get the list of types
			Type[] aTypes = theAssembly.GetTypes();

			// Add all the types to the cache
			foreach (Type type in aTypes)
			{
				AddType(Assembly.CreateQualifiedName(AssemblyPath, type.FullName), type);
			}

			return aTypes;
		}

		/// <summary>
		/// Load all of the types in the specified Assembly that implement an interface
		/// </summary>
		/// <param name="AssemblyPath"></param>
		/// <param name="Interface">Only retrieve classes that implement this interface (Case Sensitive)</param>
		/// <returns></returns>
		public IList LoadTypeList(string AssemblyPath, string Interface)
		{
			return LoadTypeList(AssemblyPath, Interface, false);
		}
		
		/// <summary>
		/// Load all of the types in the specified Assembly that implement an interface
		/// </summary>
		/// <param name="AssemblyPath"></param>
		/// <param name="Interface">Only retrieve classes that implement this interface</param>
		/// <param name="IgnoreCase"></param>
		/// <returns></returns>
		public IList LoadTypeList(string AssemblyPath, string Interface, bool IgnoreCase)
		{
			// Load the assembly into memory
			Assembly theAssembly = Assembly.LoadFrom(AssemblyPath);
			// Get the list of types
			Type[] aTypes = theAssembly.GetTypes();
			// This will hold our return
			System.Collections.ArrayList aTypesRet = new  ArrayList();

			// Add all the types to the cache
			foreach (Type type in aTypes)
			{
				object iFace = type.GetInterface(Interface, IgnoreCase);
				
				if (iFace != null)
				{
					aTypesRet.Add(type);
					AddType(Assembly.CreateQualifiedName(AssemblyPath, type.FullName), type);
				}
			}

			return aTypesRet;
		}

		/// <summary>
		/// Retrieve a specific type from an assembly.
		/// </summary>
		/// <param name="AssemblyPath"></param>
		/// <param name="Type"></param>
		/// <returns></returns>
		public System.Type LoadType(string AssemblyPath, string Type)
		{
			// Get the fully qualified name
			string strFullyQualified = Assembly.CreateQualifiedName(AssemblyPath, Type);
			// See if we have the type in our cache
			System.Type type = collTypes[strFullyQualified] as System.Type;
			if (type != null)
				return type;
			
			// We don't already have the type cached.

			// Load the assembly into memory
			Assembly theAssembly = Assembly.LoadFrom(AssemblyPath);
			// Get the list of types
			type = theAssembly.GetType(Type);
			if (type == null)
			{
				// Failed to load
				return null;
			}

			AddType(Assembly.CreateQualifiedName(AssemblyPath, type.FullName), type);

			return type;
		}

		/// <summary>
		/// An internal method for storing a type in the collection only
		/// if it does not already exist in the collection.
		/// </summary>
		/// <param name="strFullyQualified"></param>
		/// <param name="type"></param>
		private void AddType(string strFullyQualified, System.Type type)
		{
			object o = collTypes[strFullyQualified];
			if (o == null)
				collTypes.Add(strFullyQualified, type);
		}
	}
}
